@@ -5,12 +5,12 @@ This module contains 5 types to build a fuzz test: `FuzzerBuilder[1–5]`.
 
 The idea is to help you specify each part of the fuzzer progressively:
 1. the function to fuzz
-2. the [mutator](Mutator) to generate arguments to the test function (called “inputs” or “test cases”)
-3. the [serializer](Serializer) to save test cases to the file system
-4. the [sensor](Sensor) to provide feedback after running the test function, and the [pool](Pool) to interpret the feedback from the sensor
-5. [other settings](Arguments) for the fuzzer, such as the maximum allowed complexity for the test cases, where to save the corpora or artifacts on the file system, etc.
+2. the [mutator](crate::Mutator) to generate arguments to the test function (called “inputs” or “test cases”)
+3. the [serializer](crate::Serializer) to save test cases to the file system
+4. the [sensor](crate::Sensor) to provide feedback after running the test function, and the [pool](crate::Pool) to interpret the feedback from the sensor
+5. [other settings](crate::Arguments) for the fuzzer, such as the maximum allowed complexity for the test cases, where to save the corpora or artifacts on the file system, etc.
 
-In most cases, you don't need to manually specify all these components. If the argument type of the function has a [default mutator](DefaultMutator) and is serializable with serde, then you can write:
+In most cases, you don't need to manually specify all these components. If the argument type of the function has a [default mutator](crate::DefaultMutator) and is serializable with serde, then you can write:
 ```no_run
 # fn test_function(x: &bool) {}
 let _ = fuzzcheck::fuzz_test(test_function) // FuzzerBuilder1
@@ -32,7 +32,7 @@ let _ = fuzzcheck::fuzz_test(test_function)
     .launch();
 # }
 ```
-If you'd like to use a custom mutator, serializer, sensor and pool, or argumemts, you can write:
+If you'd like to use a custom mutator, serializer, sensor and pool, or arguments, you can write:
 ```no_run
 # use fuzzcheck::DefaultMutator;
 # use fuzzcheck::builder::default_sensor_and_pool;
@@ -53,32 +53,34 @@ let _ = fuzzcheck::fuzz_test(test_function)
 # }
 ```
 
-To build a custom sensor and pool, you may want to look at the [`Sensor`], [`Pool`], and [`CompatibleWithSensor`] traits.
+To build a custom sensor and pool, you may want to look at the [`Sensor`], [`Pool`](crate::Pool), and [`CompatibleWithObservations`] traits.
 You can also look at the types provided in the [`sensors_and_pools`](crate::sensors_and_pools) module. But the easiest way to customize them
-is to use the [`CodeCoverageSensorAndPoolBuilder`], although it only offers a couple limited options.
+is to use the [`SensorAndPoolBuilder`], although it only offers a couple limited options.
 */
 
-use crate::code_coverage_sensor::CodeCoverageSensor;
-use crate::fuzzer::{Fuzzer, FuzzingResult};
-use crate::sensors_and_pools::AndPool;
-use crate::sensors_and_pools::MaximiseCounterValuePool;
-use crate::sensors_and_pools::MostNDiversePool;
-use crate::sensors_and_pools::SimplestToActivateCounterPool;
-use crate::sensors_and_pools::{NumberOfActivatedCounters, OptimiseAggregateStatPool, SumOfCounterValues};
-use crate::traits::{CompatibleWithSensor, Mutator, Pool, Sensor, Serializer};
-use crate::{split_string_by_whitespace, DefaultMutator};
-
-#[cfg(feature = "serde_json_serializer")]
-use crate::SerdeSerializer;
-
-use fuzzcheck_common::arg::{options_parser, ArgumentsError, COMMAND_FUZZ, COMMAND_MINIFY_INPUT, INPUT_FILE_FLAG};
-use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
 use std::borrow::Borrow;
 use std::marker::PhantomData;
 use std::path::Path;
 use std::result::Result;
 use std::time::Duration;
 
+use fuzzcheck_common::arg::{
+    options_parser, Arguments, ArgumentsError, FuzzerCommand, COMMAND_FUZZ, COMMAND_MINIFY_INPUT, INPUT_FILE_FLAG,
+};
+
+use crate::code_coverage_sensor::CodeCoverageSensor;
+use crate::fuzzer::{Fuzzer, FuzzingResult};
+use crate::sensors_and_pools::{
+    AndPool, DifferentObservations, MaximiseEachCounterPool, MaximiseObservationPool, MostNDiversePool,
+    SameObservations, SimplestToActivateCounterPool, WrapperSensor,
+};
+#[cfg(feature = "serde_json_serializer")]
+use crate::SerdeSerializer;
+use crate::{
+    split_string_by_whitespace, CompatibleWithObservations, DefaultMutator, Mutator, PoolExt, Sensor, SensorExt,
+    Serializer,
+};
+
 /** A function that can be fuzz-tested.
 
 Strictly speaking, fuzzcheck can only test functions of type `Fn(&T) -> bool`.
@@ -163,7 +165,7 @@ where
 pub struct FuzzerBuilder2<F, M, V>
 where
     F: Fn(&V) -> bool + 'static,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
 {
     test_function: F,
@@ -173,14 +175,14 @@ where
 
 /// A fuzz-test builder that knows the function to fuzz-test, the mutator, and the serializer.
 ///
-/// Use [`self.sensor_and_pool(..)`](FuzzerBuilder3::sensor_and_pool) to specify the [sensor](Sensor) and [pool](Pool) and obtain a [FuzzerBuilder4].
+/// Use [`self.sensor_and_pool(..)`](FuzzerBuilder3::sensor_and_pool) to specify the [sensor](Sensor) and [pool](crate::Pool) and obtain a [FuzzerBuilder4].
 ///
 /// Alternatively, use [`self.default_sensor_and_pool(..)`](FuzzerBuilder3::default_sensor_and_pool) to use fuzzcheck’s
 /// default sensor, which monitors code coverage.
 pub struct FuzzerBuilder3<F, M, V>
 where
     F: Fn(&V) -> bool + 'static,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
 {
     test_function: F,
@@ -199,10 +201,10 @@ where
 pub struct FuzzerBuilder4<F, M, V, Sens, P>
 where
     F: Fn(&V) -> bool + 'static,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
     Sens: Sensor,
-    P: Pool + CompatibleWithSensor<Sens>,
+    P: CompatibleWithObservations<Sens::Observations>,
 {
     test_function: F,
     mutator: M,
@@ -228,10 +230,10 @@ where
 pub struct FuzzerBuilder5<F, M, V, Sens, P>
 where
     F: Fn(&V) -> bool + 'static,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
     Sens: Sensor,
-    P: Pool + CompatibleWithSensor<Sens>,
+    P: CompatibleWithObservations<Sens::Observations>,
 {
     test_function: F,
     mutator: M,
@@ -285,8 +287,8 @@ where
         F::NormalizedFunction,
         <T::Owned as DefaultMutator>::Mutator,
         T::Owned,
-        CodeCoverageSensor,
-        impl Pool + CompatibleWithSensor<CodeCoverageSensor>,
+        DiverseAndMaxHitsSensor,
+        BasicAndDiverseAndMaxHitsPool,
     > {
         self.mutator(<T::Owned as DefaultMutator>::default_mutator())
             .serializer(SerdeSerializer::default())
@@ -335,7 +337,7 @@ where
         }
         fn fuzz_test() {
             fuzzcheck::fuzz_test(foo)
-                .mutator(VecMutator::new(u8::default_mutator(), 2 ..= 10))
+                .mutator(VecMutator::new(u8::default_mutator(), 2 ..= 10, true))
                 // ..
                 # ;
         }
@@ -373,7 +375,7 @@ where
 impl<F, M, V> FuzzerBuilder2<F, M, V>
 where
     F: Fn(&V) -> bool,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
 {
     /**
@@ -428,13 +430,13 @@ where
 impl<F, M, V> FuzzerBuilder3<F, M, V>
 where
     F: Fn(&V) -> bool,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
 {
     #[no_coverage]
     pub fn default_sensor_and_pool(
         self,
-    ) -> FuzzerBuilder4<F, M, V, CodeCoverageSensor, impl CompatibleWithSensor<CodeCoverageSensor>> {
+    ) -> FuzzerBuilder4<F, M, V, DiverseAndMaxHitsSensor, BasicAndDiverseAndMaxHitsPool> {
         let (sensor, pool) = default_sensor_and_pool().finish();
         FuzzerBuilder4 {
             test_function: self.test_function,
@@ -446,7 +448,7 @@ where
         }
     }
     #[no_coverage]
-    pub fn sensor_and_pool<Sens: Sensor, P: CompatibleWithSensor<Sens>>(
+    pub fn sensor_and_pool<Sens: Sensor, P: CompatibleWithObservations<Sens::Observations>>(
         self,
         sensor: Sens,
         pool: P,
@@ -465,10 +467,10 @@ where
 impl<F, M, V, Sens, P> FuzzerBuilder4<F, M, V, Sens, P>
 where
     F: Fn(&V) -> bool,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
     Sens: Sensor,
-    P: Pool + CompatibleWithSensor<Sens>,
+    P: CompatibleWithObservations<Sens::Observations>,
 {
     #[no_coverage]
     pub fn arguments(self, arguments: Arguments) -> FuzzerBuilder5<F, M, V, Sens, P> {
@@ -522,7 +524,10 @@ fuzzcheck {minify} --{input_file} "artifacts/crash.json"
         let arguments = std::env::var("FUZZCHECK_ARGS").unwrap();
         let arguments = split_string_by_whitespace(&arguments);
         let matches = parser.parse(arguments).map_err(ArgumentsError::from);
-        let arguments = match matches.and_then(|matches| Arguments::from_matches(&matches, false)) {
+        let arguments = match matches.and_then(
+            #[no_coverage]
+            |matches| Arguments::from_matches(&matches, false),
+        ) {
             Ok(r) => r,
             Err(e) => {
                 println!("{}\n\n{}", e, help);
@@ -544,54 +549,62 @@ fuzzcheck {minify} --{input_file} "artifacts/crash.json"
 impl<F, M, V, Sens, P> FuzzerBuilder5<F, M, V, Sens, P>
 where
     F: Fn(&V) -> bool + 'static,
-    V: Clone,
+    V: Clone + 'static,
     M: Mutator<V>,
     Sens: Sensor + 'static,
-    P: Pool + CompatibleWithSensor<Sens> + 'static,
+    P: CompatibleWithObservations<Sens::Observations> + 'static,
     Fuzzer<V, M>: 'static,
 {
+    #[must_use]
     #[no_coverage]
     pub fn command(self, command: FuzzerCommand) -> Self {
         let mut x = self;
         x.arguments.command = command;
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn in_corpus(self, path: Option<&Path>) -> Self {
         let mut x = self;
         x.arguments.corpus_in = path.map(Path::to_path_buf);
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn out_corpus(self, path: Option<&Path>) -> Self {
         let mut x = self;
         x.arguments.corpus_out = path.map(Path::to_path_buf);
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn artifacts_folder(self, path: Option<&Path>) -> Self {
         let mut x = self;
         x.arguments.artifacts_folder = path.map(Path::to_path_buf);
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn maximum_complexity(self, max_input_cplx: f64) -> Self {
         let mut x = self;
         x.arguments.max_input_cplx = max_input_cplx;
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn stop_after_iterations(self, number_of_iterations: usize) -> Self {
         let mut x = self;
         x.arguments.maximum_iterations = number_of_iterations;
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn stop_after_duration(self, duration: Duration) -> Self {
         let mut x = self;
         x.arguments.maximum_duration = duration;
         x
     }
+    #[must_use]
     #[no_coverage]
     pub fn stop_after_first_test_failure(self, stop_after_first_test_failure: bool) -> Self {
         let mut x = self;
@@ -601,18 +614,6 @@ where
     /// Launch the fuzz test!
     #[no_coverage]
     pub fn launch(self) -> FuzzingResult<V> {
-        #[cfg(fuzzing)]
-        return self.launch_even_if_cfg_fuzzing_is_not_set();
-        #[cfg(not(fuzzing))]
-        return FuzzingResult {
-            found_test_failure: true,
-            reason_for_stopping: crate::fuzzer::ReasonForStopping::LaunchedFuzzcheckWithoutCfgFuzzing,
-        };
-    }
-
-    /// do not use
-    #[no_coverage]
-    pub fn launch_even_if_cfg_fuzzing_is_not_set(self) -> FuzzingResult<V> {
         let FuzzerBuilder5 {
             test_function,
             mutator,
@@ -633,60 +634,98 @@ where
     }
 }
 
-/// An alias for the basic pool chosen to handle the [`CodeCoverageSensor`]'s observations
-pub type BasicPool = SimplestToActivateCounterPool;
-/// An alias for the type of the pool which tries to find a fixed-length set of test cases which, together, activate the most counters.
-pub type DiversePool = AndPool<MostNDiversePool, OptimiseAggregateStatPool<NumberOfActivatedCounters>>;
-/// An alias for the type of the pool which tries to find test cases repeatedly hitting the same counters.
-pub type MaxHitsPool = AndPool<MaximiseCounterValuePool, OptimiseAggregateStatPool<SumOfCounterValues>>;
+pub type BasicSensor = CodeCoverageSensor;
+pub type DiverseSensor = impl WrapperSensor<
+    Wrapped = CodeCoverageSensor,
+    Observations = (<CodeCoverageSensor as Sensor>::Observations, usize),
+>;
+pub type MaxHitsSensor = impl WrapperSensor<
+    Wrapped = CodeCoverageSensor,
+    Observations = (<CodeCoverageSensor as Sensor>::Observations, u64),
+>;
+pub type BasicAndMaxHitsSensor = impl WrapperSensor<
+    Wrapped = CodeCoverageSensor,
+    Observations = (<CodeCoverageSensor as Sensor>::Observations, u64),
+>;
 
-/// An alias for the combination of the [`BasicPool`] and the [`DiversePool`]
-pub type BasicAndDiversePool = AndPool<SimplestToActivateCounterPool, DiversePool>;
-/// An alias for the combination of the [`BasicPool`] and the [`MaxHitsPool`]
-pub type BasicAndMaxHitsPool = AndPool<SimplestToActivateCounterPool, MaxHitsPool>;
-/// An alias for the combination of the [`BasicPool`], the [`DiversePool`], and the [`MaxHitsPool`]
-pub type BasicAndDiverseAndMaxHitsPool = AndPool<SimplestToActivateCounterPool, AndPool<DiversePool, MaxHitsPool>>;
+pub type DiverseAndMaxHitsSensor =
+    impl Sensor<Observations = (<CodeCoverageSensor as Sensor>::Observations, (usize, u64))>;
+
+pub type BasicPool = SimplestToActivateCounterPool;
+pub type DiversePool = AndPool<MostNDiversePool, MaximiseObservationPool<u64>, DifferentObservations>;
+pub type MaxHitsPool = AndPool<MaximiseEachCounterPool, MaximiseObservationPool<u64>, DifferentObservations>;
+pub type BasicAndDiversePool = AndPool<
+    AndPool<SimplestToActivateCounterPool, MostNDiversePool, SameObservations>,
+    MaximiseObservationPool<usize>,
+    DifferentObservations,
+>;
+pub type BasicAndMaxHitsPool = AndPool<
+    AndPool<SimplestToActivateCounterPool, MaximiseEachCounterPool, SameObservations>,
+    MaximiseObservationPool<u64>,
+    DifferentObservations,
+>;
+
+pub type BasicAndDiverseAndMaxHitsPool = AndPool<
+    AndPool<
+        AndPool<SimplestToActivateCounterPool, MostNDiversePool, SameObservations>,
+        MaximiseEachCounterPool,
+        SameObservations,
+    >,
+    AndPool<MaximiseObservationPool<usize>, MaximiseObservationPool<u64>, DifferentObservations>,
+    DifferentObservations,
+>;
 
 #[no_coverage]
-pub fn max_cov_hits_sensor_and_pool() -> CodeCoverageSensorAndPoolBuilder<MaxHitsPool> {
+pub fn max_cov_hits_sensor_and_pool() -> SensorAndPoolBuilder<MaxHitsSensor, MaxHitsPool> {
     let sensor = CodeCoverageSensor::observing_only_files_from_current_dir();
     let nbr_counters = sensor.count_instrumented;
-    CodeCoverageSensorAndPoolBuilder {
-        sensor,
-        pool: AndPool::new(
-            MaximiseCounterValuePool::new("max_each_cov_hits", nbr_counters),
-            OptimiseAggregateStatPool::<SumOfCounterValues>::new("max_total_cov_hits"),
-            192, // choose max_each_cov_hits ~75% of the time
-        ),
-    }
+    let sensor = sensor.map(
+        #[no_coverage]
+        |o| {
+            let sum = o
+                .iter()
+                .map(
+                    #[no_coverage]
+                    |(_, count)| count,
+                )
+                .sum::<u64>();
+            (o, sum)
+        },
+    );
+    let pool = MaximiseEachCounterPool::new("max_each_cov_hits", nbr_counters).and(
+        MaximiseObservationPool::new("max_total_cov_hits"),
+        Some(0.1),
+        DifferentObservations,
+    );
+    SensorAndPoolBuilder { sensor, pool }
 }
 
-/// Create the initial [sensor and pool builder](CodeCoverageSensorAndPoolBuilder)
+/// Create the initial [sensor and pool builder](SensorAndPoolBuilder)
 ///
-/// Use [`.find_most_diverse_set_of_test_cases()`](CodeCoverageSensorAndPoolBuilder::<BasicPool>::find_most_diverse_set_of_test_cases)
-/// or [`.find_test_cases_repeatedly_hitting_coverage_counters()`](CodeCoverageSensorAndPoolBuilder::<BasicPool>::find_test_cases_repeatedly_hitting_coverage_counters)
-/// on the result to augment the pool. Or use [`.finish()`](CodeCoverageSensorAndPoolBuilder::finish) to obtain the concrete sensor and pool.
+/// Use [`.find_most_diverse_set_of_test_cases()`](SensorAndPoolBuilder::<BasicPool>::find_most_diverse_set_of_test_cases)
+/// or [`.find_test_cases_repeatedly_hitting_coverage_counters()`](SensorAndPoolBuilder::<BasicPool>::find_test_cases_repeatedly_hitting_coverage_counters)
+/// on the result to augment the pool. Or use [`.finish()`](SensorAndPoolBuilder::finish) to obtain the concrete sensor and pool.
 #[no_coverage]
-pub fn basic_sensor_and_pool() -> CodeCoverageSensorAndPoolBuilder<BasicPool> {
+pub fn basic_sensor_and_pool() -> SensorAndPoolBuilder<BasicSensor, BasicPool> {
     let sensor = CodeCoverageSensor::observing_only_files_from_current_dir();
     let nbr_counters = sensor.count_instrumented;
-    CodeCoverageSensorAndPoolBuilder {
+    SensorAndPoolBuilder {
         sensor,
         pool: SimplestToActivateCounterPool::new("simplest_cov", nbr_counters),
     }
 }
 
-/// Create the [sensor and pool builder](CodeCoverageSensorAndPoolBuilder) that is used by default by fuzzcheck
+/// Create the [sensor and pool builder](SensorAndPoolBuilder) that is used by default by fuzzcheck
 ///
 /// Currently, the result cannot be augmented any further. Thus, the only action you can take on the result is to
-/// use [`.finish()`](CodeCoverageSensorAndPoolBuilder::finish) to obtain the concrete sensor and pool.
+/// use [`.finish()`](SensorAndPoolBuilder::finish) to obtain the concrete sensor and pool.
 #[no_coverage]
-pub fn default_sensor_and_pool() -> CodeCoverageSensorAndPoolBuilder<BasicAndDiverseAndMaxHitsPool> {
+pub fn default_sensor_and_pool() -> SensorAndPoolBuilder<DiverseAndMaxHitsSensor, BasicAndDiverseAndMaxHitsPool> {
     basic_sensor_and_pool()
         .find_most_diverse_set_of_test_cases(20)
         .find_test_cases_repeatedly_hitting_coverage_counters()
 }
-/// A builder to create a [sensor](Sensor) and [pool](Pool) that can be given as argument to
+/// A builder to create a [sensor](Sensor) and [pool](crate::Pool) that can be given as argument to
 /// [`FuzzerBuilder3::sensor_and_pool`].
 ///
 /// # Usage
@@ -698,26 +737,28 @@ pub fn default_sensor_and_pool() -> CodeCoverageSensorAndPoolBuilder<BasicAndDiv
 ///     .find_test_cases_repeatedly_hitting_coverage_counters() // optional
 ///     .finish(); // mandatory
 /// ```
-pub struct CodeCoverageSensorAndPoolBuilder<P>
+pub struct SensorAndPoolBuilder<S, P>
 where
-    P: CompatibleWithSensor<CodeCoverageSensor>,
+    S: Sensor,
+    P: CompatibleWithObservations<S::Observations>,
 {
-    sensor: CodeCoverageSensor,
+    sensor: S,
     pool: P,
 }
 
-impl<P> CodeCoverageSensorAndPoolBuilder<P>
+impl<S, P> SensorAndPoolBuilder<S, P>
 where
-    P: CompatibleWithSensor<CodeCoverageSensor>,
+    S: Sensor,
+    P: CompatibleWithObservations<S::Observations>,
 {
     /// Obtain the sensor and pool from the builder
     #[no_coverage]
-    pub fn finish(self) -> (CodeCoverageSensor, impl CompatibleWithSensor<CodeCoverageSensor>) {
+    pub fn finish(self) -> (S, P) {
         (self.sensor, self.pool)
     }
 }
 
-impl CodeCoverageSensorAndPoolBuilder<BasicPool> {
+impl SensorAndPoolBuilder<BasicSensor, BasicPool> {
     /// Augment the current pool such that it also tries to find a fixed-length set of test cases which, together,
     /// trigger the most code coverage.
     ///
@@ -727,94 +768,102 @@ impl CodeCoverageSensorAndPoolBuilder<BasicPool> {
     pub fn find_most_diverse_set_of_test_cases(
         self,
         size: usize,
-    ) -> CodeCoverageSensorAndPoolBuilder<BasicAndDiversePool> {
+    ) -> SensorAndPoolBuilder<DiverseSensor, BasicAndDiversePool> {
         let nbr_counters = self.sensor.count_instrumented;
-        CodeCoverageSensorAndPoolBuilder {
-            sensor: self.sensor,
-            pool: AndPool::new(
-                self.pool,
-                AndPool::new(
-                    MostNDiversePool::new(&format!("diverse_cov_{}", size), size, nbr_counters),
-                    OptimiseAggregateStatPool::<NumberOfActivatedCounters>::new("diverse_cov_1"),
-                    192, // choose most n diverse ~75% of the time
-                ),
-                192, // 75% if the time
-            ),
-        }
+        let sensor = self.sensor.map(
+            #[no_coverage]
+            |o| {
+                let len = o.len();
+                (o, len)
+            },
+        );
+        let pool = self
+            .pool
+            .and(
+                MostNDiversePool::new(&format!("diverse_cov_{}", size), size, nbr_counters),
+                Some(0.1),
+                SameObservations,
+            )
+            .and(
+                MaximiseObservationPool::<usize>::new("diverse_cov_1"),
+                Some(0.01),
+                DifferentObservations,
+            );
+        SensorAndPoolBuilder { sensor, pool }
     }
     /// Augment the current pool such that it also tries to find test cases repeatedly hitting the same regions of code.
     #[no_coverage]
     pub fn find_test_cases_repeatedly_hitting_coverage_counters(
         self,
-    ) -> CodeCoverageSensorAndPoolBuilder<BasicAndMaxHitsPool> {
+    ) -> SensorAndPoolBuilder<BasicAndMaxHitsSensor, BasicAndMaxHitsPool> {
         let nbr_counters = self.sensor.count_instrumented;
-        CodeCoverageSensorAndPoolBuilder {
-            sensor: self.sensor,
-            pool: AndPool::new(
-                self.pool,
-                AndPool::new(
-                    MaximiseCounterValuePool::new("max_each_cov_hits", nbr_counters),
-                    OptimiseAggregateStatPool::<SumOfCounterValues>::new("max_total_cov_hits"),
-                    192, // choose max_each_cov_hits ~75% of the time
-                ),
-                217, // ~85% of the time
-            ),
-        }
+        let sensor = self.sensor.map(
+            #[no_coverage]
+            |o| {
+                let sum = o
+                    .iter()
+                    .map(
+                        #[no_coverage]
+                        |(_, count)| count,
+                    )
+                    .sum::<u64>();
+                (o, sum)
+            },
+        );
+        let pool = self
+            .pool
+            .and(
+                MaximiseEachCounterPool::new("max_each_cov_hits", nbr_counters),
+                Some(0.1),
+                SameObservations,
+            )
+            .and(
+                MaximiseObservationPool::<u64>::new("max_total_cov_hits"),
+                Some(0.01),
+                DifferentObservations,
+            );
+        SensorAndPoolBuilder { sensor, pool }
     }
 }
-impl CodeCoverageSensorAndPoolBuilder<BasicAndDiversePool> {
+impl SensorAndPoolBuilder<DiverseSensor, BasicAndDiversePool> {
     /// Augment the current pool such that it also tries to find test cases repeatedly hitting the same regions of code.
     #[no_coverage]
     pub fn find_test_cases_repeatedly_hitting_coverage_counters(
         self,
-    ) -> CodeCoverageSensorAndPoolBuilder<BasicAndDiverseAndMaxHitsPool> {
-        let nbr_counters = self.sensor.count_instrumented;
-        CodeCoverageSensorAndPoolBuilder {
-            sensor: self.sensor,
-            pool: AndPool::new(
-                self.pool.p1, // smallest to activate counter pool
-                AndPool::new(
-                    self.pool.p2, // diverse pool
-                    AndPool::new(
-                        // in the end, the max_hits pool is chosen ~10% of the time
-                        MaximiseCounterValuePool::new("max_each_cov_hits", nbr_counters),
-                        OptimiseAggregateStatPool::<SumOfCounterValues>::new("max_total_cov_hits"),
-                        192, // choose max_each_cov_hits ~75% of the time
-                    ),
-                    183, // choose diverse pool 71% of the time => in the end, chosen (100% - 65%) * 75% = ~25% of the time
+    ) -> SensorAndPoolBuilder<DiverseAndMaxHitsSensor, BasicAndDiverseAndMaxHitsPool> {
+        let nbr_counters = self.sensor.wrapped().count_instrumented;
+
+        let sensor = self.sensor.map(
+            #[no_coverage]
+            |o| {
+                let sum =
+                    o.0.iter()
+                        .map(
+                            #[no_coverage]
+                            |(_, count)| count,
+                        )
+                        .sum::<u64>();
+                (o.0, (o.1, sum))
+            },
+        );
+
+        let pool = self
+            .pool
+            .p1
+            .and(
+                MaximiseEachCounterPool::new("max_each_cov_hits", nbr_counters),
+                Some(0.1),
+                SameObservations,
+            )
+            .and(
+                self.pool.p2.and(
+                    MaximiseObservationPool::<u64>::new("max_total_cov_hits"),
+                    Some(0.01),
+                    DifferentObservations,
                 ),
-                166, // 65% of the time
-            ),
-        }
-    }
-}
-impl CodeCoverageSensorAndPoolBuilder<BasicAndMaxHitsPool> {
-    /// Augment the current pool such that it also tries to find a fixed-length set of test cases which, together,
-    /// trigger the most code coverage.
-    ///
-    /// ### Argument
-    /// `size` : the size of the set of test cases to find
-    #[no_coverage]
-    pub fn find_most_diverse_set_of_test_cases(
-        self,
-        size: usize,
-    ) -> CodeCoverageSensorAndPoolBuilder<BasicAndDiverseAndMaxHitsPool> {
-        let nbr_counters = self.sensor.count_instrumented;
-        CodeCoverageSensorAndPoolBuilder {
-            sensor: self.sensor,
-            pool: AndPool::new(
-                self.pool.p1, // smallest to activate counter pool
-                AndPool::new(
-                    AndPool::new(
-                        MostNDiversePool::new(&format!("diverse_cov_{}", size), size, nbr_counters),
-                        OptimiseAggregateStatPool::<NumberOfActivatedCounters>::new("diverse_cov1"),
-                        192, // choose most n diverse ~75% of the time
-                    ),
-                    self.pool.p2, // diverse pool
-                    183, // choose diverse pool 71% of the time => in the end, chosen (100% - 65%) * 75% = ~25% of the time
-                ),
-                166, // 65% of the time
-            ),
-        }
+                None,
+                DifferentObservations,
+            );
+        SensorAndPoolBuilder { sensor, pool }
     }
 }
