@@ -1,12 +1,12 @@
-use super::leb128;
-use flate2::Status;
-use object::Object;
-use object::ObjectSection;
-use std::collections::HashMap;
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::convert::TryFrom;
 use std::path::{Path, PathBuf};
 
+use flate2::Status;
+use object::{Object, ObjectSection};
+
+use super::leb128;
+
 type CovMap = HashMap<[u8; 8], Vec<String>>;
 
 extern "C" {
@@ -246,8 +246,11 @@ fn read_mapping_regions(
     filename_indices: &[usize],
 ) -> Result<Vec<(RawCounter, MappingRegion)>, ReadCovMapError> {
     assert!(!covfun.is_empty());
-
     let mut result = Vec::new();
+    // the reference says we should read this number, but it doesn't actually exist
+    // and their example doesn't have it either, so I have included here and commented it out
+    // let num_regions_arrays = read_leb_usize(covfun, idx);
+    // assert_eq!(num_regions_arrays, filename_indices.len());
     for &filename_index in filename_indices {
         let num_regions = read_leb_usize(covfun, idx);
         let mut prev_line_start = 0;
@@ -256,7 +259,8 @@ fn read_mapping_regions(
             let header = read_counter(raw_header); //read_leb_usize(covfun, idx)); // counter or pseudo-counter
             match header {
                 RawCounter::Zero if raw_header != 0 => {
-                    return Err(ReadCovMapError::PseudoCountersNotSupportedYet { raw_header })
+                    // TODO: interpret the pseudo counter
+                    return Err(ReadCovMapError::PseudoCountersNotSupportedYet { raw_header });
                 }
                 _ => {}
             }
@@ -289,9 +293,22 @@ pub fn read_covfun(covfun: &[u8], idx: &mut usize) -> Result<Vec<RawFunctionCoun
     while *idx < covfun.len() {
         let function_record_header = read_first_function_record_fields(covfun, idx);
         let idx_before_encoding_data = *idx;
-        let file_id_mapping = read_file_id_mapping(covfun, idx);
-        let expressions = read_coverage_expressions(covfun, idx);
-        let counters = read_mapping_regions(covfun, idx, &file_id_mapping.filename_indices)?;
+
+        // somehow, length_encoded_data == 0 is possible!
+        let (file_id_mapping, expressions, counters) = if function_record_header.length_encoded_data == 0 {
+            (
+                FileIDMapping {
+                    filename_indices: vec![],
+                },
+                vec![],
+                vec![],
+            )
+        } else {
+            let file_id_mapping = read_file_id_mapping(covfun, idx);
+            let expressions = read_coverage_expressions(covfun, idx);
+            let counters = read_mapping_regions(covfun, idx, &file_id_mapping.filename_indices)?;
+            (file_id_mapping, expressions, counters)
+        };
 
         if idx_before_encoding_data + function_record_header.length_encoded_data != *idx {
             return Err(ReadCovMapError::InconsistentLengthOfEncodedData {
@@ -306,6 +323,9 @@ pub fn read_covfun(covfun: &[u8], idx: &mut usize) -> Result<Vec<RawFunctionCoun
         };
         *idx += padding;
 
+        if function_record_header.length_encoded_data == 0 {
+            assert_eq!(function_record_header.id.structural_hash, 0);
+        }
         if function_record_header.id.structural_hash == 0 {
             // dummy function, ignore
             continue;
@@ -328,25 +348,26 @@ pub struct PrfData {
 
 #[no_coverage]
 pub fn read_prf_data(prf_data: &[u8], idx: &mut usize) -> Result<Vec<PrfData>, ReadCovMapError> {
+    // Read the prf_data section.
+    //
+    // The problem is that there is no clear reference for it, and its format can be updated by newer LLVM versions
+    //
+    // Look at this commit: https://github.com/llvm/llvm-project/commit/24c615fa6b6b7910c8743f9044226499adfac4e6
+    // (as well as the commit it references) for a few of the files involved in generating the prf_data section, which
+    // can then be used to implement this function
+    //
+    // In particular, look at InstrProfData.inc
+
     let mut counts = Vec::new();
-    let mut prv_counter_pointer_and_nbr_counters: Option<(u64, u64)> = None;
 
     while *idx < prf_data.len() {
-        // NameRef, see InstrProdData.inc line 72
-        // IndexedInstrProf::ComputeHash(getPGOFuncNameVarInitializer(Inc->getName())
         let name_md5 = read_i64(prf_data, idx);
-        // FuncHash, see InstrProdData.inc line 75
-        // Inc->getHash()->getZExtValue()
         let structural_hash = read_u64(prf_data, idx);
-        // if structural_hash == 0 {
-        //     return Err(ReadCovMapError::FoundProfileDataForDummyFunction);
-        // }
         let function_id = FunctionIdentifier {
             name_md5,
             structural_hash,
         };
-
-        let counter_ptr = read_u64(prf_data, idx);
+        let _relative_counter_ptr = read_u64(prf_data, idx);
         let _function_ptr = read_u64(prf_data, idx);
         let _values = read_u64(prf_data, idx); // values are only used for PGO, not coverage instrumentation
 
@@ -358,25 +379,31 @@ pub fn read_prf_data(prf_data: &[u8], idx: &mut usize) -> Result<Vec<PrfData>, R
             // 1 counter seems to be the minimum for some reason
             assert!(nbr_counters <= 1);
         }
+        // u16 but aligned
+        let _num_value_site = read_i16(prf_data, idx); // this is used for PGO only, I think
+        *idx += 2; // alignment
 
-        if let Some((prv_counter_pointer, prv_nbr_counters)) = prv_counter_pointer_and_nbr_counters {
-            if prv_counter_pointer + 8 * prv_nbr_counters != counter_ptr {
-                return Err(ReadCovMapError::InconsistentCounterPointersAndLengths {
-                    prev_pointer: prv_counter_pointer as usize,
-                    length: prv_nbr_counters as usize,
-                    cur_pointer: counter_ptr as usize,
-                });
-            }
-        }
-        prv_counter_pointer_and_nbr_counters = Some((counter_ptr, nbr_counters as u64));
+        // This is no longer a valid check with LLVM 14.0, I think?
+        // Maybe due to:
+        //     https://github.com/llvm/llvm-project/commit/a1532ed27582038e2d9588108ba0fe8237f01844
+        //     https://github.com/llvm/llvm-project/commit/24c615fa6b6b7910c8743f9044226499adfac4e6
+        // if let Some((prv_counter_pointer, prv_nbr_counters)) = prv_counter_pointer_and_nbr_counters {
+        //     if prv_counter_pointer + 8 * prv_nbr_counters != counter_ptr {
+        //         return Err(
+        //             ReadCovMapError::InconsistentCounterPointersAndLengths {
+        //                 prev_pointer: prv_counter_pointer as usize,
+        //                 length: prv_nbr_counters as usize,
+        //                 cur_pointer: counter_ptr as usize,
+        //             }
+        //         );
+        //     }
+        // }
+        // prv_counter_pointer_and_nbr_counters = Some((counter_ptr, nbr_counters as u64));
 
         counts.push(PrfData {
             function_id,
             number_of_counters: nbr_counters as usize,
         });
-        // u16 but aligned
-        let _something_i_dont_know_what = read_i16(prf_data, idx); // this is used for PGO only, I think
-        *idx += 2; // alignment
     }
 
     Ok(counts)
@@ -435,7 +462,8 @@ pub fn read_prf_names(slice: &[u8], idx: &mut usize) -> Result<Vec<String>, Read
 pub struct FunctionRecord {
     pub header: FunctionRecordHeader,
     pub file_id_mapping: FileIDMapping,
-    pub expressions: Vec<(ExpandedExpression, MappingRegion)>,
+    pub expressions: Vec<(ExpandedExpression, Vec<MappingRegion>)>,
+    pub inferred_expressions: Vec<(Vec<MappingRegion>, Vec<usize>)>,
     pub name_function: String,
     pub filenames: Vec<PathBuf>,
 }
@@ -448,23 +476,140 @@ pub fn process_function_records(
 ) -> Vec<FunctionRecord> {
     let mut all_expressions = Vec::new();
     for function_counters in records {
-        let mut expressions = vec![];
-        let mut expressions_set = HashSet::new();
+        let mut expressions: Vec<(ExpandedExpression, Vec<MappingRegion>)> = vec![];
+        // map from expanded expression to an index in `expressions`
+        let mut expressions_map: HashMap<ExpandedExpression, usize> = HashMap::new();
+        let mut all_subexpressions = HashSet::new();
+
+        // NOTE: the order with which the `expressions` variable is built matters
         for (raw_counter, mapping_region) in function_counters.counters_list.iter() {
             let mut expanded = ExpandedExpression::default();
-            expanded.push_counter(raw_counter, Sign::Positive, &function_counters);
+            expanded.push_counter(raw_counter, Sign::Positive, &function_counters, &mut all_subexpressions);
             expanded.sort(); // sort them to canonicalise their representation
-            if expressions_set.contains(&expanded) {
-                continue;
+            all_subexpressions.insert(expanded.clone());
+            if let Some(idx) = expressions_map.get(&expanded) {
+                expressions[*idx].1.push(mapping_region.clone());
             } else {
-                expressions_set.insert(expanded.clone());
-                expressions.push((expanded, mapping_region.clone()));
+                expressions_map.insert(expanded.clone(), expressions.len());
+                expressions.push((expanded.clone(), vec![mapping_region.clone()]));
+            }
+        }
+        let name_function = prf_names[&function_counters.header.id.name_md5].clone();
+
+        let mut to_delete: HashMap<usize, HashSet<ExpandedExpression>> = HashMap::new();
+
+        'outer: for (i, (e1, _)) in expressions.iter().enumerate() {
+            if to_delete.contains_key(&i) {
+                continue 'outer;
+            };
+            if e1.add_terms.is_empty() {
+                continue 'outer;
+            }
+            'inner: for (j, (e2, _)) in expressions.iter().enumerate() {
+                if i == j {
+                    continue 'inner;
+                }
+                // the below means that every add_term in e1 is also included in e2
+                // in other words, the add_terms of e2 are a superset of those of e1
+                for c1 in &e1.add_terms {
+                    if !e2.add_terms.contains(c1) {
+                        continue 'inner;
+                    }
+                }
+                // the below means that every sub_term in e2 is also included in e1
+                // in other words, the sub_terms of e1 are a superset of those of e2
+
+                for c2 in &e2.sub_terms {
+                    if !e1.sub_terms.contains(c2) {
+                        continue 'inner;
+                    }
+                }
+
+                // for example
+                // e1: [1, 2, 3] [4, 5]
+                // e2: [1, 2, 3, 6] [4]
+
+                // so if e1 > 0, it follows that e2 > 0
+                // so reaching e1 is a sufficient condition to reaching e2
+
+                // we keep track of all sufficient conditions for all deleted expressions
+                let mut sufficient_expressions = vec![e1.clone()];
+                // also take into consideration the difference of sub terms between e1 and e2
+                // e.g. e1 = [c1] [c3, diff_sub_terms]
+                //      e2 = [c1, diff_add_terms] [c3]
+                // whenever e1 > 0, then e2 > 0
+                // diff_sub_terms = [diff_sub_terms]
+                // whenever e1 = 0 && diff_sub_terms > 0, then we know e2 > 0 as well
+                // otherwise, if e1 = 0 && diff_sub_terms > 0 and e2 = 0, we have:
+                // e1 = c1 - c3 - diff_sub_terms = 0 => c1 = c3 + diff_sub_terms
+                // e2 = c1 + diff_add_terms - c3 = 0 => diff_add_terms + diff_sub_terms = 0
+                // but diff_sub_terms > 0 and diff_add_terms >= 0 ! ---> contradiction
+                let mut diff_sub_terms = ExpandedExpression::default();
+                for &sub_term in &e1.sub_terms {
+                    if !e2.sub_terms.contains(&sub_term) {
+                        diff_sub_terms.add_terms.push(sub_term);
+                    }
+                }
+                'q: for (e_suff, _) in expressions.iter() {
+                    if e_suff.add_terms.is_empty() {
+                        continue 'q;
+                    }
+                    if !e_suff.sub_terms.is_empty() {
+                        continue 'q;
+                    }
+                    // all terms in e_suff.add_terms must be in diff_sub_terms.add_terms
+                    // at least one term in diff_sub_terms.add_terms must be in e_suff.add_terms
+                    // e.g.
+                    // diff_sub_terms : [c4]
+                    // e_suff : [c4]
+                    // or
+                    // diff_sub_terms: [c4, c5]
+                    // e_suff: [c5]
+                    for c1 in &e_suff.add_terms {
+                        if !diff_sub_terms.add_terms.contains(c1) {
+                            continue 'q;
+                        }
+                    }
+                    sufficient_expressions.push(e_suff.clone());
+                }
+                to_delete.entry(j).or_default().extend(sufficient_expressions);
             }
         }
 
-        let name_function = (&prf_names[&function_counters.header.id.name_md5]).clone();
+        let mut to_delete = to_delete.into_iter().collect::<Vec<_>>();
+        to_delete.sort_by(
+            #[no_coverage]
+            |a, b| b.0.cmp(&a.0),
+        );
+        let mut deleted = vec![];
+        for (e_idx, sufficient_expressions) in to_delete {
+            deleted.push((expressions.remove(e_idx).1, sufficient_expressions));
+        }
 
-        let expressions = expressions.into_iter().collect::<Vec<_>>();
+        let mut expression_to_index = HashMap::new();
+        for (i, (e, _)) in expressions.iter().enumerate() {
+            expression_to_index.insert(e, i);
+        }
+        let inferred_expressions = deleted
+            .into_iter()
+            .filter(
+                #[no_coverage]
+                |(regions, _)| !regions.is_empty(),
+            )
+            .map(
+                #[no_coverage]
+                |(regions, suff_expressions)| {
+                    let suff_expressions = suff_expressions
+                        .into_iter()
+                        .filter_map(
+                            #[no_coverage]
+                            |e| expression_to_index.get(&e).copied(),
+                        )
+                        .collect::<Vec<_>>();
+                    (regions, suff_expressions)
+                },
+            )
+            .collect::<Vec<_>>();
 
         let filenames = &covmap[&function_counters.header.hash_translation_unit];
         let mut filepaths = Vec::new();
@@ -473,11 +618,11 @@ pub fn process_function_records(
             let filepath = Path::new(filename).to_path_buf();
             filepaths.push(filepath);
         }
-
         all_expressions.push(FunctionRecord {
             header: function_counters.header,
             file_id_mapping: function_counters.file_id_mapping,
             expressions,
+            inferred_expressions,
             name_function,
             filenames: filepaths,
         });
@@ -495,11 +640,11 @@ pub enum CovMapSection {
 
 #[derive(Debug)]
 pub enum ReadCovMapError {
-    InconsistentCounterPointersAndLengths {
-        prev_pointer: usize,
-        length: usize,
-        cur_pointer: usize,
-    },
+    // InconsistentCounterPointersAndLengths {
+    //     prev_pointer: usize,
+    //     length: usize,
+    //     cur_pointer: usize,
+    // },
     // FoundProfileDataForDummyFunction,
     InconsistentLengthOfEncodedData {
         section: CovMapSection,
@@ -522,7 +667,7 @@ pub enum ReadCovMapError {
         expected: usize,
     },
     InvalidVersion(i32),
-    CannotFindFunctionRecordAssociatedWithPrfData,
+    CannotFindFunctionRecordAssociatedWithPrfData(String),
     CannotParseUTF8 {
         section: CovMapSection,
     },
@@ -630,7 +775,7 @@ pub enum RawCounter {
     Expression { operation_sign: Sign, idx: usize },
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum Sign {
     Negative,
     Positive,
@@ -684,16 +829,27 @@ impl ExpandedExpression {
         }
     }
     #[no_coverage]
-    pub fn push_counter(&mut self, c: &RawCounter, sign: Sign, ctx: &RawFunctionCounters) {
+    pub fn push_counter(
+        &mut self,
+        c: &RawCounter,
+        sign: Sign,
+        ctx: &RawFunctionCounters,
+        expressions: &mut HashSet<ExpandedExpression>,
+    ) {
         match c {
             RawCounter::Zero => {}
-            RawCounter::Counter { idx } => self.push_leaf_counter(*idx, sign),
+            RawCounter::Counter { idx } => {
+                self.push_leaf_counter(*idx, sign);
+                let mut e = ExpandedExpression::default();
+                e.add_terms.push(*idx);
+                expressions.insert(e);
+            }
             RawCounter::Expression { operation_sign, idx } => {
                 let e = &ctx.expression_list[*idx];
                 let lhs = &e.lhs;
-                self.push_counter(lhs, sign, ctx);
+                self.push_counter(lhs, sign, ctx, expressions);
                 let rhs = &e.rhs;
-                self.push_counter(rhs, sign.and(*operation_sign), ctx);
+                self.push_counter(rhs, sign.and(*operation_sign), ctx, expressions);
             }
         }
     }
@@ -704,25 +860,6 @@ impl ExpandedExpression {
     }
 }
 
-// impl FunctionRecord {
-//     #[no_coverage]
-//     pub fn filter_trivial_expressions(&mut self) -> Vec<(usize, MappingRegion)> {
-//         // filtered represent all the (unique) expressions that consist of only one positive term
-//         // that term is necessarily a reference to one of the “physical” counters in __llvm_prf_cnts
-//         let filtered = self
-//             .expressions
-//             .drain_filter(|(e, _)| e.add_terms.len() <= 1 && e.sub_terms.is_empty())
-//             .filter_map(|(e, mapping_region)| {
-//                 e.add_terms.get(0).map(
-//                     #[no_coverage]
-//                     |c| (*c, mapping_region),
-//                 )
-//             })
-//             .collect::<Vec<_>>();
-//         filtered
-//     }
-// }
-
 pub struct OptimisedExpandedExpression {
     add_terms: Vec<*const u64>,
     sub_terms: Vec<*const u64>,
@@ -746,12 +883,6 @@ impl OptimisedExpandedExpression {
 impl ExpandedExpression {
     #[no_coverage]
     fn optimised(&self, counters: &[u64]) -> OptimisedExpandedExpression {
-        // assert!(
-        //     self.add_terms.len() > 1 || !self.sub_terms.is_empty(),
-        //     "{} {}",
-        //     self.add_terms.len(),
-        //     self.sub_terms.len()
-        // );
         let mut add_terms = Vec::new();
         let mut sub_terms = Vec::new();
         for &add_term in &self.add_terms {
@@ -772,18 +903,8 @@ pub struct Coverage {
     pub counters_len: usize,
     pub single_counters: Vec<*mut u64>,
     pub expression_counters: Vec<OptimisedExpandedExpression>,
-    // pub(crate) computed_expressions: Vec<u64>,
 }
 
-// impl Coverage {
-//     #[no_coverage]
-//     pub(crate) fn compute_expressions(&mut self) {
-//         for expr_counter in &self.expression_counters {
-//             self.computed_expressions.push(expr_counter.compute());
-//         }
-//     }
-// }
-
 impl Coverage {
     #[no_coverage]
     pub fn new(
@@ -805,7 +926,7 @@ impl Coverage {
                             #[no_coverage]
                             |fr| fr.header.id == prf_data.function_id,
                         )
-                        .ok_or(ReadCovMapError::CannotFindFunctionRecordAssociatedWithPrfData)?;
+                        .ok_or_else(|| ReadCovMapError::CannotFindFunctionRecordAssociatedWithPrfData("This can sometimes happen when the crate is incrementally compiled with more than one codegen-unit. Try adding codegen-units = 1 or incremental = false to the appropriate profile (release or fuzzing) in Cargo.toml".to_string()))?;
 
                     let slice = &mut all_counters[range];
                     let mut single_counters = Vec::new();
@@ -841,13 +962,13 @@ impl Coverage {
     where
         F: Fn(&Path) -> bool,
     {
-        all_self.drain_filter(
+        all_self.retain(
             #[no_coverage]
             |coverage| {
                 if let Some(filepath) = coverage.function_record.filenames.first() {
-                    !keep_f(filepath)
+                    keep_f(filepath)
                 } else {
-                    false
+                    true
                 }
             },
         );
