@@ -2,13 +2,11 @@
 
 extern crate fastrand;
 
-use std::cmp::Ordering;
-use std::cmp::PartialOrd;
+use std::cmp::{Ordering, PartialOrd};
 use std::fmt;
 use std::fmt::Debug;
 use std::hash::Hash;
-use std::ops::Index;
-use std::ops::IndexMut;
+use std::ops::{Index, IndexMut};
 
 // ========= Slab ============
 
@@ -83,8 +81,15 @@ pub struct Slab<T> {
 }
 
 impl<T: Debug> Debug for Slab<T> {
+    #[no_coverage]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let storage = self.keys().map(|k| &self[k]).collect::<Vec<_>>();
+        let storage = self
+            .keys()
+            .map(
+                #[no_coverage]
+                |k| &self[k],
+            )
+            .collect::<Vec<_>>();
         f.debug_struct("Slab")
             .field("storage", &storage)
             .field("available_slots", &self.available_slots.len())
@@ -205,8 +210,15 @@ pub struct RcSlab<T> {
 }
 
 impl<T: Debug> Debug for RcSlab<T> {
+    #[no_coverage]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let storage = self.keys().map(|k| &self[k]).collect::<Vec<_>>();
+        let storage = self
+            .keys()
+            .map(
+                #[no_coverage]
+                |k| &self[k],
+            )
+            .collect::<Vec<_>>();
         f.debug_struct("Slab")
             .field("storage", &storage)
             .field("available_slots", &self.available_slots.len())
@@ -279,6 +291,29 @@ impl<T> RcSlab<T> {
             Some(&mut self.storage[key].data)
         }
     }
+
+    #[no_coverage]
+    pub fn get_mut_and_ref(&mut self, key1: usize, key2: usize) -> Option<(&mut T, &T)> {
+        if key1 == key2 {
+            panic!("key1 must be different than key2");
+        }
+        if self.available_slots.contains(&key1) || self.available_slots.contains(&key2) {
+            None
+        } else {
+            if key1 < key2 {
+                let (slice1, slice2) = self.storage.split_at_mut(key1 + 1);
+                let a = &mut slice1[key1].data;
+                let b = &mut slice2[key2 - key1 - 1].data;
+                Some((a, b))
+            } else {
+                // key1 > key2
+                let (slice1, slice2) = self.storage.split_at_mut(key2 + 1);
+                let b = &mut slice1[key2].data;
+                let a = &mut slice2[key1 - key2 - 1].data;
+                Some((a, b))
+            }
+        }
+    }
 }
 
 impl<T> Index<usize> for RcSlab<T> {
