@@ -1,20 +1,25 @@
+use std::any::{Any, TypeId};
+use std::borrow::Borrow;
+use std::collections::hash_map::DefaultHasher;
+use std::collections::HashMap;
+use std::hash::{Hash, Hasher};
+use std::panic::{catch_unwind, AssertUnwindSafe};
+use std::process::exit;
+use std::result::Result;
+
+use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
+use fuzzcheck_common::{FuzzerEvent, FuzzerStats};
+use libc::{SIGABRT, SIGALRM, SIGBUS, SIGFPE, SIGINT, SIGSEGV, SIGTERM, SIGTRAP};
+
 use crate::data_structures::RcSlab;
 use crate::sensors_and_pools::{
     AndSensorAndPool, NoopSensor, TestFailure, TestFailurePool, TestFailureSensor, UnitPool, TEST_FAILURE,
 };
 use crate::signals_handler::set_signal_handlers;
+use crate::subvalue_provider::{CrossoverSubValueProvider, Generation, SubValueProviderId};
 use crate::traits::{CorpusDelta, Mutator, SaveToStatsFolder, SensorAndPool, Serializer};
 use crate::world::World;
-use crate::{CSVField, FuzzedInput, ToCSV};
-use fuzzcheck_common::arg::{Arguments, FuzzerCommand};
-use fuzzcheck_common::{FuzzerEvent, FuzzerStats};
-use libc::{SIGABRT, SIGALRM, SIGBUS, SIGFPE, SIGINT, SIGSEGV, SIGTERM, SIGTRAP};
-use std::borrow::Borrow;
-use std::collections::hash_map::DefaultHasher;
-use std::hash::{Hash, Hasher};
-use std::panic::{catch_unwind, AssertUnwindSafe};
-use std::process::exit;
-use std::result::Result;
+use crate::{CSVField, SubValueProvider, ToCSV};
 
 static WRITE_STATS_ERROR: &str = "the stats could not be written to the file system";
 static WORLD_NEW_ERROR: &str = "an IO operation failed when setting up the fuzzer";
@@ -46,7 +51,6 @@ pub enum ReasonForStopping<T> {
     ExhaustedAllPossibleMutations,
     MaxIterationsReached,
     MaxDurationReached,
-    LaunchedFuzzcheckWithoutCfgFuzzing,
 }
 
 /// The index to a test case in the fuzzer’s storage.
@@ -54,8 +58,9 @@ pub enum ReasonForStopping<T> {
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub struct PoolStorageIndex(usize);
 
-#[cfg(test)]
+// #[cfg(test)]
 impl PoolStorageIndex {
+    #[no_coverage]
     pub fn mock(idx: usize) -> Self {
         Self(idx)
     }
@@ -67,10 +72,85 @@ enum FuzzerInputIndex<T> {
     Pool(PoolStorageIndex),
 }
 
-struct FuzzerState<T: Clone, M: Mutator<T>> {
+struct FuzzedInputAndSubValueProvider<T, M>
+where
+    T: Clone + 'static,
+    M: Mutator<T>,
+{
+    input: FuzzedInput<T, M>,
+    subvalues: CrossoverSubValueProvider<T, M>,
+}
+
+/**
+ * A struct that stores the value, cache, and mutation step of an input.
+ * It is used for convenience.
+ */
+struct FuzzedInput<T: Clone + 'static, Mut: Mutator<T>> {
+    value: T,
+    cache: Mut::Cache,
+    mutation_step: Mut::MutationStep,
+    generation: Generation,
+}
+impl<T: Clone + 'static, Mut: Mutator<T>> Clone for FuzzedInput<T, Mut> {
+    fn clone(&self) -> Self {
+        Self {
+            value: self.value.clone(),
+            cache: self.cache.clone(),
+            mutation_step: self.mutation_step.clone(),
+            generation: self.generation,
+        }
+    }
+}
+
+impl<T: Clone + 'static, Mut: Mutator<T>> FuzzedInput<T, Mut> {
+    #[no_coverage]
+    fn new(value: T, cache: Mut::Cache, mutation_step: Mut::MutationStep, generation: Generation) -> Self {
+        Self {
+            value,
+            cache,
+            mutation_step,
+            generation,
+        }
+    }
+
+    #[no_coverage]
+    fn new_source(&self, m: &Mut, generation: Generation) -> Self {
+        let cache = m.validate_value(&self.value).unwrap();
+        let mutation_step = m.default_mutation_step(&self.value, &cache);
+        Self::new(self.value.clone(), cache, mutation_step, generation)
+    }
+
+    #[no_coverage]
+    fn complexity(&self, m: &Mut) -> f64 {
+        m.complexity(&self.value, &self.cache)
+    }
+
+    #[no_coverage]
+    fn mutate(
+        &mut self,
+        m: &Mut,
+        subvalue_provider: &dyn SubValueProvider,
+        max_cplx: f64,
+    ) -> Option<(Mut::UnmutateToken, f64)> {
+        m.ordered_mutate(
+            &mut self.value,
+            &mut self.cache,
+            &mut self.mutation_step,
+            subvalue_provider,
+            max_cplx,
+        )
+    }
+
+    #[no_coverage]
+    fn unmutate(&mut self, m: &Mut, t: Mut::UnmutateToken) {
+        m.unmutate(&mut self.value, &mut self.cache, t);
+    }
+}
+
+struct FuzzerState<T: Clone + 'static, M: Mutator<T>> {
     mutator: M,
     sensor_and_pool: Box<dyn SensorAndPool>,
-    pool_storage: RcSlab<FuzzedInput<T, M>>,
+    pool_storage: RcSlab<FuzzedInputAndSubValueProvider<T, M>>,
     /// The step given to the mutator when the fuzzer wants to create a new arbitrary test case
     arbitrary_step: M::ArbitraryStep,
     /// The index of the test case that is being tested
@@ -82,18 +162,26 @@ struct FuzzerState<T: Clone, M: Mutator<T>> {
     serializer: Box<dyn Serializer<Value = T>>,
     /// The world handles effects
     world: World,
+    rng: fastrand::Rng,
 }
 
-impl<T: Clone, M: Mutator<T>> FuzzerState<T, M> {
+impl<T: Clone + 'static, M: Mutator<T>> Drop for FuzzerState<T, M> {
+    #[no_coverage]
+    fn drop(&mut self) {
+        unsafe { crate::signals_handler::reset_signal_handlers() };
+    }
+}
+
+impl<T: Clone + 'static, M: Mutator<T>> FuzzerState<T, M> {
     #[no_coverage]
     fn get_input<'a>(
         fuzzer_input_idx: &'a FuzzerInputIndex<FuzzedInput<T, M>>,
-        pool_storage: &'a RcSlab<FuzzedInput<T, M>>,
+        pool_storage: &'a RcSlab<FuzzedInputAndSubValueProvider<T, M>>,
     ) -> Option<&'a FuzzedInput<T, M>> {
         match fuzzer_input_idx {
             FuzzerInputIndex::None => None,
             FuzzerInputIndex::Temporary(input) => Some(input),
-            FuzzerInputIndex::Pool(idx) => Some(&pool_storage[idx.0]),
+            FuzzerInputIndex::Pool(idx) => Some(&pool_storage[idx.0].input),
         }
     }
 }
@@ -103,15 +191,16 @@ fn update_fuzzer_stats(stats: &mut FuzzerStats, world: &mut World) {
     let microseconds = world.elapsed_time_since_last_checkpoint();
     let nbr_runs = stats.total_number_of_runs - stats.number_of_runs_since_last_reset_time;
     let nbr_runs_times_million = nbr_runs * 1_000_000;
-    stats.exec_per_s = nbr_runs_times_million / microseconds;
-
+    if microseconds != 0 {
+        stats.exec_per_s = nbr_runs_times_million / microseconds;
+    }
     if microseconds > 1_000_000 {
         world.set_checkpoint_instant();
         stats.number_of_runs_since_last_reset_time = stats.total_number_of_runs;
     }
 }
 
-impl<T: Clone, M: Mutator<T>> SaveToStatsFolder for FuzzerState<T, M>
+impl<T: Clone + 'static, M: Mutator<T>> SaveToStatsFolder for FuzzerState<T, M>
 where
     Self: 'static,
 {
@@ -123,7 +212,7 @@ where
     }
 }
 
-impl<T: Clone, M: Mutator<T>> FuzzerState<T, M>
+impl<T: Clone + 'static, M: Mutator<T>> FuzzerState<T, M>
 where
     Self: 'static,
 {
@@ -142,6 +231,7 @@ where
         match signal {
             SIGABRT | SIGBUS | SIGSEGV | SIGFPE | SIGALRM | SIGTRAP => {
                 if let Some(input) = Self::get_input(&self.input_idx, &self.pool_storage) {
+                    let input = input.new_source(&self.mutator, Generation(0));
                     let cplx = input.complexity(&self.mutator);
                     let content = self.serializer.to_data(&input.value);
                     let _ = self.world.save_artifact(content, cplx, self.serializer.extension());
@@ -170,7 +260,7 @@ where
         {
             let cache = self.mutator.validate_value(&v).unwrap();
             let step = self.mutator.default_mutation_step(&v, &cache);
-            Some((FuzzedInput::new(v, cache, step, 0), cplx))
+            Some((FuzzedInput::new(v, cache, step, Generation(0)), cplx))
         } else {
             None
         }
@@ -180,14 +270,14 @@ where
         let ptr = self as *mut Self;
         set_signal_handlers(
             #[no_coverage]
-            move |sig| (&mut *ptr).receive_signal(sig),
+            move |sig| (*ptr).receive_signal(sig),
         );
     }
 }
 
 pub struct Fuzzer<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
     Self: 'static,
 {
@@ -197,7 +287,7 @@ where
 
 impl<T, M> Fuzzer<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
     Self: 'static,
 {
@@ -222,6 +312,7 @@ where
                 settings,
                 serializer,
                 world,
+                rng: fastrand::Rng::new(),
             },
             test,
         }
@@ -239,6 +330,7 @@ where
                     fuzzer_stats,
                     serializer,
                     world,
+                    settings,
                     ..
                 },
             test,
@@ -261,12 +353,27 @@ where
                 }
             },
         ));
+        if settings.detect_infinite_loop {
+            let _old_time_left = unsafe { libc::alarm(1) };
+            // TODO: I think setitimer should be prefered, but libc
+            // doesn't support it on linux, see:
+            // https://github.com/rust-lang/libc/issues/1347#event-3879031340
 
+            // let success = unsafe {
+            // let t = itimerval {
+            //     it_interval: libc::timeval { tv_sec: 0, tv_usec: 0 },
+            //     it_value: libc::timeval { tv_sec: 1, tv_usec: 0 },
+            // };
+            // libc::setitimer(ITIMER_REAL, &t, std::ptr::null_mut())
+            // };
+            // assert!(success == 0);
+        }
         sensor_and_pool.start_recording();
         let result = catch_unwind(AssertUnwindSafe(
             #[no_coverage]
             || (test)(input.value.borrow()),
         ));
+
         let _ = std::panic::take_hook();
         let test_failure = match result {
             Ok(false) => unsafe {
@@ -322,12 +429,41 @@ where
                 .expect(UPDATE_CORPUS_ERROR);
             world.report_event(event, Some((fuzzer_stats, sensor_and_pool.stats().as_ref())));
             if add_ref_count > 0 {
-                let new_input = input.new_source(mutator);
-                // here I don't check the complexity of the new input,
-                // but because of the way mutators work (real possibility of
-                // inconsistent complexities), then its complexity may be higher
-                // than the maximum allowed one
-                pool_storage.insert(new_input, add_ref_count);
+                let generation = Generation(fuzzer_stats.total_number_of_runs);
+                let input = input.new_source(mutator, generation);
+                // check that the mutator's handling of the complexity is correct
+                assert!(
+                    (input.complexity(mutator) - cplx).abs() < 0.01,
+                    "The mutator used by the fuzz test does not evaluate the complexity of the test cases consistently.
+                    This is a bug in the implementation of {}",
+                    std::any::type_name::<M>()
+                );
+
+                let mut subvalues: HashMap<TypeId, Vec<(*const dyn Any, f64)>> = HashMap::default();
+                mutator.visit_subvalues(
+                    &input.value,
+                    &input.cache,
+                    #[no_coverage]
+                    &mut |subvalue, complexity| {
+                        subvalues
+                            .entry(subvalue.type_id())
+                            .or_default()
+                            .push((subvalue as *const _, complexity));
+                    },
+                );
+                let storage_idx_1 = pool_storage.next_slot();
+                let subvalues = CrossoverSubValueProvider::new(
+                    SubValueProviderId {
+                        idx: storage_idx_1,
+                        generation,
+                    },
+                    &input.value,
+                    &input.cache,
+                    mutator,
+                );
+                let stored_input = FuzzedInputAndSubValueProvider { input, subvalues };
+                let storage_idx_2 = pool_storage.insert(stored_input, add_ref_count);
+                assert_eq!(storage_idx_1, storage_idx_2);
             }
             for delta in deltas {
                 for r in delta.remove {
@@ -339,6 +475,26 @@ where
         Ok(())
     }
 
+    #[no_coverage]
+    fn get_input_and_subvalue_provider<'a>(
+        pool_storage: &'a mut RcSlab<FuzzedInputAndSubValueProvider<T, M>>,
+        sensor_and_pool: &mut dyn SensorAndPool,
+        rng: &fastrand::Rng,
+        idx: PoolStorageIndex,
+    ) -> (&'a mut FuzzedInput<T, M>, &'a (impl SubValueProvider + 'a)) {
+        let idx_cross = sensor_and_pool.get_random_index().unwrap();
+
+        if idx == idx_cross || rng.u8(..5) == 0 {
+            let FuzzedInputAndSubValueProvider { input, subvalues } = &mut pool_storage[idx.0];
+            (input, subvalues)
+        } else {
+            // crossover of two different test cases
+            let (input, FuzzedInputAndSubValueProvider { subvalues, .. }) =
+                pool_storage.get_mut_and_ref(idx.0, idx_cross.0).unwrap();
+            (&mut input.input, subvalues)
+        }
+    }
+
     #[no_coverage]
     fn process_next_input(&mut self) -> Result<(), ReasonForStopping<T>> {
         let FuzzerState {
@@ -347,48 +503,54 @@ where
             input_idx,
             mutator,
             settings,
+            rng,
+            fuzzer_stats,
+            world,
             ..
         } = &mut self.state;
-        loop {
-            if let Some(idx) = sensor_and_pool.get_random_index() {
-                *input_idx = FuzzerInputIndex::Pool(idx);
-                let input = &mut pool_storage[idx.0];
-                let generation = input.generation;
-                if let Some((unmutate_token, cplx)) = input.mutate(mutator, settings.max_input_cplx) {
-                    if cplx < self.state.settings.max_input_cplx {
-                        self.test_and_process_input(cplx)?;
-                    }
 
-                    // Retrieving the input may fail because the input may have been deleted
-                    if let Some(input) = self.state.pool_storage.get_mut(idx.0) {
-                        if input.generation == generation {
-                            input.unmutate(&self.state.mutator, unmutate_token);
-                        }
-                    }
-
-                    break Ok(());
-                } else {
-                    self.state.world.report_event(
-                        FuzzerEvent::End,
-                        Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
-                    );
-                    break Err(ReasonForStopping::ExhaustedAllPossibleMutations);
-                }
-            } else if let Some((input, cplx)) = self.state.arbitrary_input() {
-                self.state.input_idx = FuzzerInputIndex::Temporary(input);
-
-                if cplx < self.state.settings.max_input_cplx {
-                    self.test_and_process_input(cplx)?;
+        if let Some(idx) = sensor_and_pool.get_random_index() {
+            *input_idx = FuzzerInputIndex::Pool(idx);
+            let (input, subvalue_provider) =
+                Self::get_input_and_subvalue_provider(pool_storage, sensor_and_pool.as_mut(), rng, idx);
+            let generation = input.generation;
+            if let Some((unmutate_token, complexity)) =
+                input.mutate(mutator, subvalue_provider, settings.max_input_cplx)
+            {
+                //drop(subvalue_provider);
+                if complexity < self.state.settings.max_input_cplx {
+                    self.test_and_process_input(complexity)?;
                 }
 
-                break Ok(());
+                // Retrieving the input may fail because the input may have been deleted
+                if let Some(input) = self.state.pool_storage.get_mut(idx.0).map(
+                    #[no_coverage]
+                    |x| &mut x.input,
+                ) {
+                    if input.generation == generation {
+                        input.unmutate(&self.state.mutator, unmutate_token);
+                    }
+                }
+
+                Ok(())
             } else {
-                self.state.world.report_event(
-                    FuzzerEvent::End,
-                    Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
-                );
-                break Err(ReasonForStopping::ExhaustedAllPossibleMutations);
+                world.report_event(FuzzerEvent::End, Some((fuzzer_stats, sensor_and_pool.stats().as_ref())));
+                Err(ReasonForStopping::ExhaustedAllPossibleMutations)
             }
+        } else if let Some((input, cplx)) = self.state.arbitrary_input() {
+            self.state.input_idx = FuzzerInputIndex::Temporary(input);
+
+            if cplx < self.state.settings.max_input_cplx {
+                self.test_and_process_input(cplx)?;
+            }
+
+            Ok(())
+        } else {
+            self.state.world.report_event(
+                FuzzerEvent::End,
+                Some((&self.state.fuzzer_stats, self.state.sensor_and_pool.stats().as_ref())),
+            );
+            Err(ReasonForStopping::ExhaustedAllPossibleMutations)
         }
     }
 
@@ -406,7 +568,7 @@ where
                     let value = self.state.serializer.from_data(&value)?;
                     let cache = self.state.mutator.validate_value(&value)?;
                     let mutation_step = self.state.mutator.default_mutation_step(&value, &cache);
-                    Some(FuzzedInput::new(value, cache, mutation_step, 0))
+                    Some(FuzzedInput::new(value, cache, mutation_step, Generation(0)))
                 },
             )
             .collect();
@@ -418,10 +580,9 @@ where
                 break;
             }
         }
-
-        inputs.drain_filter(
+        inputs.retain(
             #[no_coverage]
-            |i| i.complexity(&self.state.mutator) > self.state.settings.max_input_cplx,
+            |i| i.complexity(&self.state.mutator) <= self.state.settings.max_input_cplx,
         );
         // assert!(!inputs.is_empty());
 
@@ -488,7 +649,7 @@ pub fn launch<T, M>(
     mut args: Arguments,
 ) -> FuzzingResult<T>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
     Fuzzer<T, M>: 'static,
 {
@@ -498,8 +659,12 @@ where
             if !args.stop_after_first_failure {
                 let test_failure_sensor = TestFailureSensor::default();
                 let test_failure_pool = TestFailurePool::new("test_failures");
-                let sensor_and_pool =
-                    AndSensorAndPool::new(sensor_and_pool, Box::new((test_failure_sensor, test_failure_pool)), 254);
+                let sensor_and_pool = AndSensorAndPool::new(
+                    sensor_and_pool,
+                    Box::new((test_failure_sensor, test_failure_pool)),
+                    10.0,
+                    1.0,
+                );
                 let mut fuzzer = Fuzzer::new(
                     test,
                     mutator,
@@ -558,7 +723,12 @@ where
 
                 let noop_sensor = NoopSensor;
                 let unit_pool = UnitPool::new(PoolStorageIndex(0));
-                let sensor_and_pool = AndSensorAndPool::new(sensor_and_pool, Box::new((noop_sensor, unit_pool)), 128);
+                let sensor_and_pool =
+                    // 100:1 might seem like an excessive ratio, but the second pool will never make progress,
+                    // therefore its relative weight willl diminish over time
+                    // if after 100 iterations, the first pool makes progress, then the ratio will be 1:1
+                    // what the exact value should be and how the ratio should evolve is an open question to me
+                    AndSensorAndPool::new(sensor_and_pool, Box::new((noop_sensor, unit_pool)), 1.0, 100.0);
                 let mut fuzzer = Fuzzer::new(
                     test,
                     mutator,
@@ -567,10 +737,37 @@ where
                     args.clone(),
                     world,
                 );
-                fuzzer
-                    .state
-                    .pool_storage
-                    .insert(FuzzedInput::new(value, cache, mutation_step, 0), 1);
+
+                let mut subvalues: HashMap<TypeId, Vec<(*const dyn Any, f64)>> = HashMap::default();
+                fuzzer.state.mutator.visit_subvalues(
+                    &value,
+                    &cache,
+                    #[no_coverage]
+                    &mut |subvalue, complexity| {
+                        subvalues
+                            .entry(subvalue.type_id())
+                            .or_default()
+                            .push((subvalue as *const _, complexity));
+                    },
+                );
+                let storage_idx_1 = fuzzer.state.pool_storage.next_slot();
+                let generation = Generation(0);
+                let subvalues = CrossoverSubValueProvider::new(
+                    SubValueProviderId {
+                        idx: storage_idx_1,
+                        generation,
+                    },
+                    &value,
+                    &cache,
+                    &fuzzer.state.mutator,
+                );
+                let stored_input = FuzzedInputAndSubValueProvider {
+                    input: FuzzedInput::new(value, cache, mutation_step, generation),
+                    subvalues,
+                };
+                let storage_idx_2 = fuzzer.state.pool_storage.insert(stored_input, 1);
+
+                assert_eq!(storage_idx_1, storage_idx_2);
 
                 unsafe { fuzzer.state.set_up_signal_handler() };
 
@@ -588,9 +785,25 @@ where
             let value = serializer.from_data(&value).expect(SERIALIZER_FROM_DATA_ERROR);
             if let Some(cache) = mutator.validate_value(&value) {
                 let mutation_step = mutator.default_mutation_step(&value, &cache);
-                let input = FuzzedInput::new(value, cache, mutation_step, 0);
+                let input = FuzzedInput::new(value, cache, mutation_step, Generation(0));
                 let cplx = input.complexity(&mutator);
 
+                if args.detect_infinite_loop {
+                    let _old_time_left = unsafe { libc::alarm(1) };
+                    // TODO: I think setitimer should be prefered, but libc
+                    // doesn't support it on linux, see:
+                    // https://github.com/rust-lang/libc/issues/1347#event-3879031340
+
+                    // let success = unsafe {
+                    // let t = itimerval {
+                    //     it_interval: libc::timeval { tv_sec: 0, tv_usec: 0 },
+                    //     it_value: libc::timeval { tv_sec: 1, tv_usec: 0 },
+                    // };
+                    // libc::setitimer(ITIMER_REAL, &t, std::ptr::null_mut())
+                    // };
+                    // assert!(success == 0);
+                }
+
                 let result = catch_unwind(AssertUnwindSafe(
                     #[no_coverage]
                     || (test)(input.value.borrow()),
