@@ -1,5 +1,8 @@
+use std::any::Any;
+use std::cmp::Ordering;
+use std::marker::PhantomData;
+
 use crate::Mutator;
-use std::{cmp::Ordering, marker::PhantomData};
 
 /**
 A mutator that wraps multiple different mutators of the same type.
@@ -11,44 +14,45 @@ use fuzzcheck::mutators::integer_within_range::U8WithinRangeMutator;
 let m1 = U8WithinRangeMutator::new(3 ..= 10);
 let m2 = U8WithinRangeMutator::new(78 ..= 200);
 
-let m = AlternationMutator::new(vec![m1, m2]);
+let m = AlternationMutator::new(vec![m1, m2], 0.0);
 
 // m will produce values either in 3..=10 or in 78..=200
 ```
 */
 pub struct AlternationMutator<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
 {
     mutators: Vec<M>,
-    complexity_from_choice: f64,
-    max_complexity: f64,
-    min_complexity: f64,
     rng: fastrand::Rng,
+    search_space_complexity: f64,
+    added_complexity: f64,
+    min_complexity: f64,
+    max_complexity: f64,
     _phantom: PhantomData<T>,
 }
 
 impl<T, M> AlternationMutator<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
 {
     #[no_coverage]
-    pub fn new(mutators: Vec<M>) -> Self {
+    pub fn new(mutators: Vec<M>, added_complexity: f64) -> Self {
         assert!(!mutators.is_empty());
         let complexity_from_choice = crate::mutators::size_to_cplxity(mutators.len());
 
-        let max_complexity = mutators
+        let search_space_complexity = mutators
             .iter()
             .map(
                 #[no_coverage]
                 |m| {
-                    let max_cplx = m.max_complexity();
-                    if max_cplx == 0. {
+                    let cplx = m.global_search_space_complexity();
+                    if cplx == 0. {
                         complexity_from_choice
                     } else {
-                        max_cplx
+                        cplx
                     }
                 },
             )
@@ -57,31 +61,37 @@ where
                 |x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal),
             )
             .unwrap();
+
+        let max_complexity = mutators
+            .iter()
+            .map(
+                #[no_coverage]
+                |m| m.max_complexity() + added_complexity,
+            )
+            .max_by(
+                #[no_coverage]
+                |x1, x2| x1.partial_cmp(x2).unwrap_or(Ordering::Equal),
+            )
+            .unwrap();
         let min_complexity = mutators
             .iter()
             .map(
                 #[no_coverage]
-                |m| {
-                    let min_cplx = m.min_complexity();
-                    if min_cplx == 0. {
-                        complexity_from_choice
-                    } else {
-                        min_cplx
-                    }
-                },
+                |m| m.min_complexity() + added_complexity,
             )
             .min_by(
                 #[no_coverage]
-                |x, y| x.partial_cmp(y).unwrap_or(Ordering::Equal),
+                |x1, x2| x1.partial_cmp(x2).unwrap_or(Ordering::Equal),
             )
             .unwrap();
-        let complexity_from_choice = crate::mutators::size_to_cplxity(mutators.len());
+
         Self {
             mutators,
-            complexity_from_choice,
-            max_complexity,
-            min_complexity,
+            search_space_complexity,
             rng: fastrand::Rng::default(),
+            added_complexity,
+            min_complexity,
+            max_complexity,
             _phantom: PhantomData,
         }
     }
@@ -95,13 +105,6 @@ pub struct ArbitraryStep<AS> {
     idx: usize,
 }
 
-#[doc(hidden)]
-#[derive(Clone)]
-pub struct RecursingPartIndex<RPI> {
-    inner: Vec<RPI>,
-    indices: Vec<usize>,
-}
-
 #[doc(hidden)]
 #[derive(Clone)]
 pub struct MutationStep<MS, AS> {
@@ -124,47 +127,20 @@ pub enum UnmutateToken<T, U> {
     Inner(usize, U),
 }
 
-// impl<T, M> AlternationMutator<T, M>
-// where
-//     T: Clone,
-//     M: Mutator<T>,
-// {
-//     #[no_coverage]
-//     fn default_mutation_step(
-//         &self,
-//         inner: M::MutationStep,
-//         idx: usize,
-//     ) -> MutationStep<M::MutationStep, <Self as Mutator<T>>::ArbitraryStep> {
-//         MutationStep {
-//             step: 0,
-//             mutator_idx: idx,
-//             inner,
-//             arbitrary: {
-//                 let mut step = self.default_arbitrary_step();
-//                 step.indices.remove(idx);
-//                 step
-//             },
-//         }
-//     }
-// }
 impl<T, M> AlternationMutator<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
 {
     #[no_coverage]
     fn complexity_from_inner(&self, cplx: f64) -> f64 {
-        if cplx == 0. {
-            self.complexity_from_choice
-        } else {
-            cplx
-        }
+        cplx + self.added_complexity
     }
 }
 
 impl<T, M> Mutator<T> for AlternationMutator<T, M>
 where
-    T: Clone + 'static,
+    T: Clone + 'static + 'static,
     M: Mutator<T>,
 {
     #[doc(hidden)]
@@ -193,6 +169,17 @@ where
         }
     }
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn is_valid(&self, value: &T) -> bool {
+        for m in self.mutators.iter() {
+            if m.is_valid(value) {
+                return true;
+            }
+        }
+        false
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &T) -> Option<Self::Cache> {
@@ -211,37 +198,45 @@ where
             Some(caches)
         }
     }
+
     #[doc(hidden)]
     #[no_coverage]
     fn default_mutation_step(&self, value: &T, cache: &Self::Cache) -> Self::MutationStep {
         cache
             .iter()
-            .map(|c| {
-                let m = &self.mutators[c.mutator_idx];
-                MutationStep {
-                    step: 0,
-                    mutator_idx: c.mutator_idx,
-                    inner: m.default_mutation_step(value, &c.inner),
-                    arbitrary: {
-                        let mut step = self.default_arbitrary_step();
-                        step.indices.remove(c.mutator_idx);
-                        step
-                    },
-                }
-            })
+            .map(
+                #[no_coverage]
+                |c| {
+                    let m = &self.mutators[c.mutator_idx];
+                    MutationStep {
+                        step: 0,
+                        mutator_idx: c.mutator_idx,
+                        inner: m.default_mutation_step(value, &c.inner),
+                        arbitrary: {
+                            let mut step = self.default_arbitrary_step();
+                            step.indices.remove(c.mutator_idx);
+                            step
+                        },
+                    }
+                },
+            )
             .collect()
     }
 
+    fn global_search_space_complexity(&self) -> f64 {
+        self.search_space_complexity
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
-        self.complexity_from_inner(self.max_complexity)
+        self.max_complexity
     }
 
     #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
-        self.complexity_from_inner(self.min_complexity)
+        self.min_complexity
     }
 
     #[doc(hidden)]
@@ -290,6 +285,7 @@ where
         value: &mut T,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
         if max_cplx < self.min_complexity() {
@@ -308,6 +304,9 @@ where
         let step_idx = self.rng.usize(..step.len());
         let chosen_step = &mut step[step_idx];
         chosen_step.step += 1;
+        // TODO: instead of 20, should be the sum of all important arbitraries of the sub mutators
+        // and maybe it shouldn't be done all at the beginning, but be interspersed in between the
+        // important mutations of the current mutator
         if chosen_step.step < 20 {
             if let Some((mut v, cplx)) = self.ordered_arbitrary(&mut chosen_step.arbitrary, max_cplx) {
                 std::mem::swap(value, &mut v);
@@ -316,15 +315,25 @@ where
         }
 
         let mutator_idx = chosen_step.mutator_idx;
-        let chosen_cache = cache.iter_mut().find(|c| c.mutator_idx == mutator_idx).unwrap();
+        let chosen_cache = cache
+            .iter_mut()
+            .find(
+                #[no_coverage]
+                |c| c.mutator_idx == mutator_idx,
+            )
+            .unwrap();
 
         let idx = chosen_cache.mutator_idx;
         assert_eq!(idx, mutator_idx);
 
         let mutator = &self.mutators[idx];
-        if let Some((t, cplx)) =
-            mutator.ordered_mutate(value, &mut chosen_cache.inner, &mut chosen_step.inner, max_cplx)
-        {
+        if let Some((t, cplx)) = mutator.ordered_mutate(
+            value,
+            &mut chosen_cache.inner,
+            &mut chosen_step.inner,
+            subvalue_provider,
+            max_cplx,
+        ) {
             Some((UnmutateToken::Inner(idx, t), self.complexity_from_inner(cplx)))
         } else {
             if let Some((mut v, cplx)) = self.ordered_arbitrary(&mut chosen_step.arbitrary, max_cplx) {
@@ -335,7 +344,7 @@ where
                 if step.is_empty() {
                     None
                 } else {
-                    self.ordered_mutate(value, cache, step, max_cplx)
+                    self.ordered_mutate(value, cache, step, subvalue_provider, max_cplx)
                 }
             }
         }
@@ -353,6 +362,7 @@ where
         // the same thing every time
         // there should be a better way to prevent this though
         // maybe it's time to give random_mutate a MutationStep too?
+        // TODO: should use the global search space complexity here instead of max complexity?
         if self.rng.usize(..100) == 0 || mutator.max_complexity() < 0.1 {
             let (new_value, cplx) = self.random_arbitrary(max_cplx);
             let old_value = ::std::mem::replace(value, new_value);
@@ -372,54 +382,25 @@ where
             }
             UnmutateToken::Inner(idx, t) => {
                 let mutator = &self.mutators[idx];
-                let cache = cache.iter_mut().find(|c| c.mutator_idx == idx).unwrap();
+                let cache = cache
+                    .iter_mut()
+                    .find(
+                        #[no_coverage]
+                        |c| c.mutator_idx == idx,
+                    )
+                    .unwrap();
                 mutator.unmutate(value, &mut cache.inner, t);
             }
         }
     }
 
-    #[doc(hidden)]
-    type RecursingPartIndex = RecursingPartIndex<M::RecursingPartIndex>;
-
     #[doc(hidden)]
     #[no_coverage]
-    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
-        Self::RecursingPartIndex {
-            inner: cache
-                .iter()
-                .map(|c| self.mutators[c.mutator_idx].default_recursing_part_index(value, &c.inner))
-                .collect(),
-            indices: cache.iter().map(|c| c.mutator_idx).collect(),
-        }
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>,
-    {
-        // if vector is empty, return Nlne
-        if index.inner.is_empty() {
-            return None;
-        }
-
-        // choose a recursing part randomly
-        let choice = self.rng.usize(..index.inner.len());
-        let subindex = &mut index.inner[choice];
-        let mutator_index = index.indices[choice];
-
-        // call it
-        let result = self.mutators[mutator_index].recursing_part::<V, N>(parent, value, subindex);
-
-        // if none, remove it from the vector and call the function again
-        if result.is_none() {
-            index.inner.remove(choice);
-            index.indices.remove(choice);
-            self.recursing_part::<V, N>(parent, value, index)
-        } else {
-            result
+    fn visit_subvalues<'a>(&self, value: &'a T, cache: &'a Self::Cache, visit: &mut dyn FnMut(&'a dyn Any, f64)) {
+        for cache in cache.iter() {
+            let mutator_idx = cache.mutator_idx;
+            let mutator = &self.mutators[mutator_idx];
+            mutator.visit_subvalues(value, &cache.inner, visit);
         }
     }
 }
