@@ -1,5 +1,6 @@
-use crate::DefaultMutator;
-use crate::Mutator;
+use std::any::Any;
+
+use crate::{DefaultMutator, Mutator};
 
 /// Default mutator for `bool`
 #[derive(Default)]
@@ -50,14 +51,8 @@ impl Mutator<bool> for BoolMutator {
 
     #[doc(hidden)]
     #[no_coverage]
-    fn max_complexity(&self) -> f64 {
-        BOOL_COMPLEXITY
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn min_complexity(&self) -> f64 {
-        BOOL_COMPLEXITY
+    fn is_valid(&self, _value: &bool) -> bool {
+        true
     }
 
     #[doc(hidden)]
@@ -71,6 +66,22 @@ impl Mutator<bool> for BoolMutator {
     fn default_mutation_step(&self, _value: &bool, _cache: &Self::Cache) -> Self::MutationStep {
         INITIAL_MUTATION_STEP
     }
+
+    #[doc(hidden)]
+    #[no_coverage]
+    fn global_search_space_complexity(&self) -> f64 {
+        1.0
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn max_complexity(&self) -> f64 {
+        BOOL_COMPLEXITY
+    }
+    #[doc(hidden)]
+    #[no_coverage]
+    fn min_complexity(&self) -> f64 {
+        BOOL_COMPLEXITY
+    }
     #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, _value: &bool, _cache: &Self::Cache) -> f64 {
@@ -106,6 +117,7 @@ impl Mutator<bool> for BoolMutator {
         value: &mut bool,
         _cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        _subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
         if max_cplx < self.min_complexity() {
@@ -118,34 +130,21 @@ impl Mutator<bool> for BoolMutator {
             None
         }
     }
+
     #[doc(hidden)]
     #[no_coverage]
     fn random_mutate(&self, value: &mut bool, _cache: &mut Self::Cache, _max_cplx: f64) -> (Self::UnmutateToken, f64) {
         (std::mem::replace(value, !*value), BOOL_COMPLEXITY)
     }
+
     #[doc(hidden)]
     #[no_coverage]
     fn unmutate(&self, value: &mut bool, _cache: &mut Self::Cache, t: Self::UnmutateToken) {
         *value = t;
     }
 
-    #[doc(hidden)]
-    type RecursingPartIndex = ();
     #[doc(hidden)]
     #[no_coverage]
-    fn default_recursing_part_index(&self, _value: &bool, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, T, M>(
-        &self,
-        _parent: &M,
-        _value: &'a bool,
-        _index: &mut Self::RecursingPartIndex,
-    ) -> Option<&'a T>
-    where
-        T: Clone,
-        M: Mutator<T>,
-    {
-        None
+    fn visit_subvalues<'a>(&self, _value: &'a bool, _cache: &'a Self::Cache, _visit: &mut dyn FnMut(&'a dyn Any, f64)) {
     }
 }
