@@ -1,7 +1,9 @@
+use std::any::Any;
 use std::ops::{Range, RangeInclusive};
 
 use fastrand::Rng;
 
+use super::size_to_cplxity;
 use crate::Mutator;
 
 /// Mutator for a `char` within a list of ranges
@@ -10,12 +12,16 @@ pub struct CharacterMutator {
     ranges: Vec<RangeInclusive<char>>,
     total_length: u32,
     lengths: Vec<Range<u32>>,
-    cplx: f64,
+    search_space_complexity: f64,
+    max_cplx: f64,
+    min_cplx: f64,
     rng: Rng,
 }
 impl CharacterMutator {
     #[no_coverage]
     pub fn new(ranges: Vec<RangeInclusive<char>>) -> Self {
+        let ranges = ranges.into_iter().filter(|r| !r.is_empty()).collect::<Vec<_>>();
+        assert!(!ranges.is_empty());
         let total_length = ranges.iter().fold(
             0,
             #[no_coverage]
@@ -34,13 +40,26 @@ impl CharacterMutator {
                 },
             )
             .collect::<Vec<_>>();
-        let cplx = crate::mutators::size_to_cplxity(total_length as usize);
+        let min_cplx = if total_length == 1 {
+            let c = *ranges[0].start();
+            Self::complexity_of_value(c)
+        } else {
+            7.0
+        };
+        let max_cplx = if total_length == 1 {
+            let c = *ranges[0].start();
+            Self::complexity_of_value(c)
+        } else {
+            32.0
+        };
         let rng = Rng::new();
         Self {
             ranges,
             total_length,
             lengths,
-            cplx,
+            search_space_complexity: size_to_cplxity(total_length as usize),
+            max_cplx,
+            min_cplx,
             rng,
         }
     }
@@ -55,7 +74,16 @@ impl CharacterMutator {
         panic!()
     }
 }
-
+impl CharacterMutator {
+    fn complexity_of_value(c: char) -> f64 {
+        let cplx = (c.len_utf8() * 8) as f64;
+        if c.is_ascii_alphabetic() {
+            cplx - 1.0
+        } else {
+            cplx
+        }
+    }
+}
 impl Mutator<char> for CharacterMutator {
     #[doc(hidden)]
     type Cache = ();
@@ -65,11 +93,22 @@ impl Mutator<char> for CharacterMutator {
     type ArbitraryStep = u64;
     #[doc(hidden)]
     type UnmutateToken = char;
+
     #[doc(hidden)]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         0
     }
+
+    #[doc(hidden)]
+    #[no_coverage]
+    fn is_valid(&self, value: &char) -> bool {
+        self.ranges.iter().any(
+            #[no_coverage]
+            |range| range.contains(value),
+        )
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &char) -> Option<Self::Cache> {
@@ -88,20 +127,26 @@ impl Mutator<char> for CharacterMutator {
         0
     }
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn global_search_space_complexity(&self) -> f64 {
+        self.search_space_complexity
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
-        self.cplx
+        self.max_cplx
     }
     #[doc(hidden)]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
-        self.cplx
+        self.min_cplx
     }
     #[doc(hidden)]
     #[no_coverage]
-    fn complexity(&self, _value: &char, _cache: &Self::Cache) -> f64 {
-        self.cplx
+    fn complexity(&self, value: &char, _cache: &Self::Cache) -> f64 {
+        Self::complexity_of_value(*value)
     }
     #[doc(hidden)]
     #[no_coverage]
@@ -113,7 +158,7 @@ impl Mutator<char> for CharacterMutator {
         *step += 1;
 
         if let Some(c) = self.get_char(idx) {
-            Some((c, self.cplx))
+            Some((c, Self::complexity_of_value(c)))
         } else {
             self.ordered_arbitrary(step, max_cplx)
         }
@@ -123,7 +168,7 @@ impl Mutator<char> for CharacterMutator {
     fn random_arbitrary(&self, max_cplx: f64) -> (char, f64) {
         let idx = self.rng.u32(..self.total_length);
         if let Some(c) = self.get_char(idx) {
-            (c, self.cplx)
+            (c, Self::complexity_of_value(c))
         } else {
             self.random_arbitrary(max_cplx)
         }
@@ -135,6 +180,7 @@ impl Mutator<char> for CharacterMutator {
         value: &mut char,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
         if *step == self.total_length as u64 {
@@ -145,9 +191,9 @@ impl Mutator<char> for CharacterMutator {
 
         if let Some(mut c) = self.get_char(idx) {
             std::mem::swap(value, &mut c);
-            Some((c, self.cplx))
+            Some((c, Self::complexity_of_value(*value)))
         } else {
-            self.ordered_mutate(value, cache, step, max_cplx)
+            self.ordered_mutate(value, cache, step, subvalue_provider, max_cplx)
         }
     }
     #[doc(hidden)]
@@ -156,7 +202,7 @@ impl Mutator<char> for CharacterMutator {
         let idx = self.rng.u32(..self.total_length);
         if let Some(mut c) = self.get_char(idx) {
             std::mem::swap(value, &mut c);
-            (c, self.cplx)
+            (c, Self::complexity_of_value(*value))
         } else {
             self.random_mutate(value, cache, max_cplx)
         }
@@ -167,30 +213,16 @@ impl Mutator<char> for CharacterMutator {
         *value = t;
     }
 
-    #[doc(hidden)]
-    type RecursingPartIndex = ();
     #[doc(hidden)]
     #[no_coverage]
-    fn default_recursing_part_index(&self, _value: &char, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(
-        &self,
-        _parent: &N,
-        _value: &'a char,
-        _index: &mut Self::RecursingPartIndex,
-    ) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V> + 'static,
-    {
-        None
+    fn visit_subvalues<'a>(&self, _value: &'a char, _cache: &'a Self::Cache, _visit: &mut dyn FnMut(&'a dyn Any, f64)) {
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{mutators::character_classes::CharacterMutator, Mutator};
+    use crate::mutators::character_classes::CharacterMutator;
+    use crate::Mutator;
 
     #[test]
     #[no_coverage]
