@@ -1,3 +1,5 @@
+use std::any::Any;
+
 use crate::Mutator;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
@@ -7,7 +9,7 @@ pub enum Either<L, R> {
 }
 impl<T, M1, M2> Mutator<T> for Either<M1, M2>
 where
-    T: Clone,
+    T: Clone + 'static,
     M1: Mutator<T>,
     M2: Mutator<T>,
 {
@@ -21,6 +23,7 @@ where
     type UnmutateToken = Either<M1::UnmutateToken, M2::UnmutateToken>;
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
         match self {
@@ -30,6 +33,17 @@ where
     }
 
     #[doc(hidden)]
+    #[inline]
+    #[no_coverage]
+    fn is_valid(&self, value: &T) -> bool {
+        match self {
+            Either::Left(m) => m.is_valid(value),
+            Either::Right(m) => m.is_valid(value),
+        }
+    }
+
+    #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn validate_value(&self, value: &T) -> Option<Self::Cache> {
         match self {
@@ -43,18 +57,30 @@ where
             }
         }
     }
+
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn default_mutation_step(&self, value: &T, cache: &Self::Cache) -> Self::MutationStep {
         match (self, cache) {
             (Either::Left(m), Either::Left(c)) => Either::Left(m.default_mutation_step(value, c)),
             (Either::Right(m), Either::Right(c)) => Either::Right(m.default_mutation_step(value, c)),
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
     #[no_coverage]
+    fn global_search_space_complexity(&self) -> f64 {
+        match self {
+            Either::Left(m) => m.global_search_space_complexity(),
+            Either::Right(m) => m.global_search_space_complexity(),
+        }
+    }
+
+    #[doc(hidden)]
+    #[inline]
+    #[no_coverage]
     fn max_complexity(&self) -> f64 {
         match self {
             Either::Left(m) => m.max_complexity(),
@@ -63,6 +89,7 @@ where
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn min_complexity(&self) -> f64 {
         match self {
@@ -72,26 +99,29 @@ where
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
         match (self, cache) {
             (Either::Left(m), Either::Left(c)) => m.complexity(value, c),
             (Either::Right(m), Either::Right(c)) => m.complexity(value, c),
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
         match (self, step) {
             (Either::Left(m), Either::Left(s)) => m.ordered_arbitrary(s, max_cplx),
             (Either::Right(m), Either::Right(s)) => m.ordered_arbitrary(s, max_cplx),
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {
         match self {
@@ -101,28 +131,31 @@ where
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn ordered_mutate(
         &self,
         value: &mut T,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
         match (self, cache, step) {
             (Either::Left(m), Either::Left(c), Either::Left(s)) => {
-                let (t, cplx) = m.ordered_mutate(value, c, s, max_cplx)?;
+                let (t, cplx) = m.ordered_mutate(value, c, s, subvalue_provider, max_cplx)?;
                 Some((Either::Left(t), cplx))
             }
             (Either::Right(m), Either::Right(c), Either::Right(s)) => {
-                let (t, cplx) = m.ordered_mutate(value, c, s, max_cplx)?;
+                let (t, cplx) = m.ordered_mutate(value, c, s, subvalue_provider, max_cplx)?;
                 Some((Either::Right(t), cplx))
             }
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn random_mutate(&self, value: &mut T, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
         match (self, cache) {
@@ -134,11 +167,12 @@ where
                 let (t, cplx) = m.random_mutate(value, c, max_cplx);
                 (Either::Right(t), cplx)
             }
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
+    #[inline]
     #[no_coverage]
     fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
         match (self, cache, t) {
@@ -148,32 +182,22 @@ where
             (Either::Right(m), Either::Right(c), Either::Right(t)) => {
                 m.unmutate(value, c, t);
             }
-            _ => panic!(),
+            _ => unreachable!(),
         }
     }
 
     #[doc(hidden)]
-    type RecursingPartIndex = Either<M1::RecursingPartIndex, M2::RecursingPartIndex>;
-    #[doc(hidden)]
+    #[inline]
     #[no_coverage]
-    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
+    fn visit_subvalues<'a>(&self, value: &'a T, cache: &'a Self::Cache, visit: &mut dyn FnMut(&'a dyn Any, f64)) {
         match (self, cache) {
-            (Either::Left(m), Either::Left(c)) => Either::Left(m.default_recursing_part_index(value, c)),
-            (Either::Right(m), Either::Right(c)) => Either::Right(m.default_recursing_part_index(value, c)),
+            (Either::Left(m), Either::Left(cache)) => {
+                m.visit_subvalues(value, cache, visit);
+            }
+            (Either::Right(m), Either::Right(cache)) => {
+                m.visit_subvalues(value, cache, visit);
+            }
             _ => unreachable!(),
         }
     }
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>,
-    {
-        match (self, index) {
-            (Either::Left(m), Either::Left(i)) => m.recursing_part::<V, N>(parent, value, i),
-            (Either::Right(m), Either::Right(i)) => m.recursing_part::<V, N>(parent, value, i),
-            _ => panic!(),
-        }
-    }
 }
