@@ -1,27 +1,25 @@
+use std::any::Any;
 use std::marker::PhantomData;
 
-use crate::Mutator;
 use fastrand::Rng;
 
-#[doc(hidden)]
-#[derive(Clone)]
-pub struct RecursingPartIndex<RPI> {
-    inner: Vec<RPI>,
-    indices: Vec<usize>,
-}
+use super::CrossoverStep;
+use crate::{Mutator, CROSSOVER_RATE};
 
 /// A mutator for vectors of a specific length
 ///
 /// A different mutator is used for each element of the vector
 pub struct FixedLenVecMutator<T, M>
 where
-    T: Clone,
+    T: Clone + 'static,
     M: Mutator<T>,
 {
     pub rng: Rng,
     mutators: Vec<M>,
     min_complexity: f64,
     max_complexity: f64,
+    search_space_complexity: f64,
+    inherent_complexity: f64,
     _phantom: PhantomData<T>,
 }
 impl<T, M> FixedLenVecMutator<T, M>
@@ -31,7 +29,7 @@ where
 {
     #[no_coverage]
     pub fn new_with_repeated_mutator(mutator: M, len: usize) -> Self {
-        Self::new(vec![mutator; len])
+        Self::new(vec![mutator; len], true)
     }
 }
 
@@ -41,24 +39,52 @@ where
     M: Mutator<T>,
 {
     #[no_coverage]
-    pub fn new(mutators: Vec<M>) -> Self {
+    pub fn new(mutators: Vec<M>, inherent_complexity: bool) -> Self {
         assert!(!mutators.is_empty());
-        let max_complexity = mutators.iter().fold(0.0, |cplx, m| cplx + m.max_complexity());
-        let min_complexity = mutators.iter().fold(0.0, |cplx, m| cplx + m.min_complexity());
+
+        // NOTE: this agrees with the vector mutator
+        let inherent_complexity = if inherent_complexity {
+            1.0 + if mutators[0].min_complexity() == 0.0 {
+                mutators.len() as f64
+            } else {
+                0.0
+            }
+        } else {
+            0.0
+        };
+
+        let max_complexity = mutators.iter().fold(
+            0.0,
+            #[no_coverage]
+            |cplx, m| cplx + m.max_complexity(),
+        ) + inherent_complexity;
+        let min_complexity = mutators.iter().fold(
+            0.0,
+            #[no_coverage]
+            |cplx, m| cplx + m.min_complexity(),
+        ) + inherent_complexity;
+        let search_space_complexity = mutators.iter().fold(
+            0.0,
+            #[no_coverage]
+            |cplx, m| cplx + m.global_search_space_complexity(),
+        );
         Self {
             rng: Rng::default(),
             mutators,
             min_complexity,
             max_complexity,
+            search_space_complexity,
+            inherent_complexity,
             _phantom: PhantomData,
         }
     }
 }
 
 #[derive(Clone)]
-pub struct MutationStep<S> {
+pub struct MutationStep<T, S> {
     inner: Vec<S>,
     element_step: usize,
+    crossover_steps: Vec<CrossoverStep<T>>,
 }
 
 #[derive(Clone)]
@@ -76,7 +102,8 @@ impl<C> Default for VecMutatorCache<C> {
     }
 }
 
-pub enum UnmutateVecToken<T: Clone, M: Mutator<T>> {
+pub enum UnmutateVecToken<T: Clone + 'static, M: Mutator<T>> {
+    ReplaceElement(usize, T),
     Element(usize, M::UnmutateToken),
     Elements(Vec<(usize, M::UnmutateToken)>),
     Replace(Vec<T>),
@@ -90,7 +117,7 @@ impl<T: Clone + 'static, M: Mutator<T>> FixedLenVecMutator<T, M> {
     #[no_coverage]
     fn mutate_elements(
         &self,
-        value: &mut Vec<T>,
+        value: &mut [T],
         cache: &mut VecMutatorCache<M::Cache>,
         idcs: &[usize],
         current_cplx: f64,
@@ -115,9 +142,10 @@ impl<T: Clone + 'static, M: Mutator<T>> FixedLenVecMutator<T, M> {
     #[no_coverage]
     fn mutate_element(
         &self,
-        value: &mut Vec<T>,
+        value: &mut [T],
         cache: &mut VecMutatorCache<M::Cache>,
-        step: &mut MutationStep<M::MutationStep>,
+        step: &mut MutationStep<T, M::MutationStep>,
+        subvalue_provider: &dyn crate::SubValueProvider,
         idx: usize,
         current_cplx: f64,
         spare_cplx: f64,
@@ -129,7 +157,9 @@ impl<T: Clone + 'static, M: Mutator<T>> FixedLenVecMutator<T, M> {
 
         let old_cplx = mutator.complexity(el, el_cache);
 
-        if let Some((token, new_cplx)) = mutator.ordered_mutate(el, el_cache, el_step, spare_cplx + old_cplx) {
+        if let Some((token, new_cplx)) =
+            mutator.ordered_mutate(el, el_cache, el_step, subvalue_provider, spare_cplx + old_cplx)
+        {
             Some((
                 UnmutateVecToken::Element(idx, token),
                 current_cplx - old_cplx + new_cplx,
@@ -157,7 +187,7 @@ impl<T: Clone + 'static, M: Mutator<T>> FixedLenVecMutator<T, M> {
             remaining_cplx -= x_cplx;
             remaining_min_complexity -= mutator.min_complexity();
         }
-        (v, self.min_complexity + sum_cplx)
+        (v, sum_cplx)
     }
 }
 
@@ -165,7 +195,7 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
     #[doc(hidden)]
     type Cache = VecMutatorCache<M::Cache>;
     #[doc(hidden)]
-    type MutationStep = MutationStep<M::MutationStep>;
+    type MutationStep = MutationStep<T, M::MutationStep>;
     #[doc(hidden)]
     type ArbitraryStep = ();
     #[doc(hidden)]
@@ -175,6 +205,20 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
     #[no_coverage]
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep {}
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn is_valid(&self, value: &Vec<T>) -> bool {
+        if value.len() != self.mutators.len() {
+            return false;
+        }
+        for (m, v) in self.mutators.iter().zip(value.iter()) {
+            if !m.is_valid(v) {
+                return false;
+            }
+        }
+        true
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn validate_value(&self, value: &Vec<T>) -> Option<Self::Cache> {
@@ -215,7 +259,17 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
                 |((v, c), m)| m.default_mutation_step(v, c),
             )
             .collect::<Vec<_>>();
-        MutationStep { inner, element_step: 0 }
+        MutationStep {
+            inner,
+            element_step: 0,
+            crossover_steps: vec![CrossoverStep::default(); value.len()],
+        }
+    }
+
+    #[doc(hidden)]
+    #[no_coverage]
+    fn global_search_space_complexity(&self) -> f64 {
+        self.search_space_complexity
     }
 
     #[doc(hidden)]
@@ -233,7 +287,7 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
     #[doc(hidden)]
     #[no_coverage]
     fn complexity(&self, _value: &Vec<T>, cache: &Self::Cache) -> f64 {
-        cache.sum_cplx
+        cache.sum_cplx + self.inherent_complexity
     }
 
     #[doc(hidden)]
@@ -249,7 +303,8 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
     #[no_coverage]
     fn random_arbitrary(&self, max_cplx: f64) -> (Vec<T>, f64) {
         let target_cplx = crate::mutators::gen_f64(&self.rng, 1.0..max_cplx);
-        self.new_input_with_complexity(target_cplx)
+        let (v, sum_cplx) = self.new_input_with_complexity(target_cplx);
+        (v, sum_cplx + self.inherent_complexity)
     }
 
     #[doc(hidden)]
@@ -259,6 +314,7 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
         value: &mut Vec<T>,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
         if max_cplx < self.min_complexity() {
@@ -269,8 +325,23 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
             std::mem::swap(value, &mut v);
             return Some((UnmutateVecToken::Replace(v), cplx));
         }
+        if self.rng.u8(..CROSSOVER_RATE) == 0 {
+            let choice = self.rng.usize(..value.len());
+            let step = &mut step.crossover_steps[choice];
+            let old_el_cplx = self.mutators[choice].complexity(&value[choice], &cache.inner[choice]);
+            let current_cplx = self.complexity(value, cache);
+            let max_el_cplx = current_cplx - old_el_cplx - self.inherent_complexity;
+            if let Some((el, new_el_cplx)) = step.get_next_subvalue(subvalue_provider, max_el_cplx) {
+                if self.mutators[choice].is_valid(el) {
+                    let mut el = el.clone();
+                    std::mem::swap(&mut value[choice], &mut el);
+                    let cplx = cache.sum_cplx - old_el_cplx + new_el_cplx + self.inherent_complexity;
+                    let token = UnmutateVecToken::ReplaceElement(choice, el);
+                    return Some((token, cplx));
+                }
+            }
+        }
         let current_cplx = self.complexity(value, cache);
-        let spare_cplx = max_cplx - current_cplx;
         if value.len() > 1 && self.rng.usize(..20) == 0 {
             let mut idcs = (0..value.len()).collect::<Vec<_>>();
             self.rng.shuffle(&mut idcs);
@@ -278,9 +349,10 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
             let idcs = &idcs[..count];
             Some(self.mutate_elements(value, cache, idcs, current_cplx, max_cplx))
         } else {
+            let spare_cplx = max_cplx - current_cplx - self.inherent_complexity;
             let idx = step.element_step % value.len();
             step.element_step += 1;
-            self.mutate_element(value, cache, step, idx, current_cplx, spare_cplx)
+            self.mutate_element(value, cache, step, subvalue_provider, idx, current_cplx, spare_cplx)
                 .or_else(
                     #[no_coverage]
                     || Some(self.random_mutate(value, cache, max_cplx)),
@@ -336,60 +408,31 @@ impl<T: Clone + 'static, M: Mutator<T>> Mutator<Vec<T>> for FixedLenVecMutator<T
             UnmutateVecToken::Replace(new_value) => {
                 let _ = std::mem::replace(value, new_value);
             }
+            UnmutateVecToken::ReplaceElement(idx, el) => {
+                let _ = std::mem::replace(&mut value[idx], el);
+            }
         }
     }
 
-    #[doc(hidden)]
-    type RecursingPartIndex = RecursingPartIndex<M::RecursingPartIndex>;
     #[doc(hidden)]
     #[no_coverage]
-    fn default_recursing_part_index(&self, value: &Vec<T>, cache: &Self::Cache) -> Self::RecursingPartIndex {
-        RecursingPartIndex {
-            inner: value
-                .iter()
-                .zip(cache.inner.iter())
-                .zip(self.mutators.iter())
-                .map(|((v, c), m)| m.default_recursing_part_index(v, c))
-                .collect(),
-            indices: (0..value.len()).collect(),
-        }
-    }
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(
-        &self,
-        parent: &N,
-        value: &'a Vec<T>,
-        index: &mut Self::RecursingPartIndex,
-    ) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>,
-    {
-        assert_eq!(index.inner.len(), index.indices.len());
-        if index.inner.is_empty() {
-            return None;
-        }
-        let choice = self.rng.usize(..index.inner.len());
-        let subindex = &mut index.inner[choice];
-        let value_index = index.indices[choice];
-        let v = &value[value_index];
-        let result = self.mutators[value_index].recursing_part(parent, v, subindex);
-        if result.is_none() {
-            index.inner.remove(choice);
-            index.indices.remove(choice);
-            self.recursing_part::<V, N>(parent, value, index)
-        } else {
-            result
+    fn visit_subvalues<'a>(&self, value: &'a Vec<T>, cache: &'a Self::Cache, visit: &mut dyn FnMut(&'a dyn Any, f64)) {
+        if !value.is_empty() {
+            for idx in 0..value.len() {
+                let cplx = self.mutators[idx].complexity(&value[idx], &cache.inner[idx]);
+                visit(&value[idx], cplx);
+            }
+            for ((el, el_cache), mutator) in value.iter().zip(cache.inner.iter()).zip(self.mutators.iter()) {
+                mutator.visit_subvalues(el, el_cache, visit);
+            }
         }
     }
 }
 #[cfg(test)]
 mod tests {
-    use crate::Mutator;
-
     use super::FixedLenVecMutator;
     use crate::mutators::integer::U8Mutator;
+    use crate::Mutator;
     #[test]
     #[no_coverage]
     fn test_constrained_length_mutator() {
