@@ -1,9 +1,9 @@
 //! Grammar-based mutators and related utilties.
 //!
-//! This module provides two mutators a grammar-based `impl Mutator<AST>` which generates an abstract syntax
-//! tree satisfying a grammar, created though [`grammar_based_ast_mutator`]. You can then obtain a string
-//! from the [`AST`] by calling [`ast.to_string()`](AST::to_string).
-//!
+//! This module provides a grammar-based `impl Mutator<AST>` which generates an abstract syntax
+//! tree satisfying a grammar, created through [`grammar_based_ast_mutator`]. The resulting mutator can be
+//! transformed into a `Mutator<(AST, String)>`, where the second element of the tuple is the string corresponding
+//! to the abstract syntax tree, by calling [`.with_string()`](ASTMutator::with_string).
 //!
 //! To specify a grammar, you should use the following functions:
 #![cfg_attr(
@@ -64,30 +64,22 @@ let rule = recursive(|rule| {
 
 mod ast;
 mod grammar;
-// mod incremental_map_conformance;
-// mod list;
 mod mutators;
-// mod parser;
 
 #[cfg(feature = "regex_grammar")]
 mod regex;
 
 #[doc(inline)]
 pub use ast::AST;
-#[doc(inline)]
-pub use grammar::Grammar;
-#[doc(inline)]
-pub use grammar::{alternation, concatenation, literal, literal_range, literal_ranges, recurse, recursive, repetition};
-
 #[cfg(feature = "regex_grammar")]
 #[doc(inline)]
 #[doc(cfg(feature = "regex_grammar"))]
 pub use grammar::regex;
-
+#[doc(inline)]
+pub use grammar::Grammar;
+#[doc(inline)]
+pub use grammar::{alternation, concatenation, literal, literal_range, literal_ranges, recurse, recursive, repetition};
 #[doc(inline)]
 pub use mutators::grammar_based_ast_mutator;
-
 #[doc(inline)]
-// pub use mutators::grammar_based_string_mutator;
-pub use mutators::GrammarBasedASTMutator;
-// pub use mutators::GrammarBasedStringMutator;
+pub use mutators::ASTMutator;
