@@ -1,6 +1,6 @@
-use crate::DefaultMutator;
-use crate::Mutator;
+use std::any::Any;
 
+use crate::{DefaultMutator, Mutator};
 /*
     These mutators try to achieve multiple things:
     * avoid repetitions, such that if the value “7” was already produced, then it will not appear again
@@ -158,6 +158,7 @@ macro_rules! impl_int_mutator {
             type ArbitraryStep = u64;
             #[doc(hidden)]
             type UnmutateToken = $name; // old value
+
             #[doc(hidden)]
             #[no_coverage]
             fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
@@ -165,6 +166,11 @@ macro_rules! impl_int_mutator {
             }
             #[doc(hidden)]
             #[no_coverage]
+            fn is_valid(&self, _value: &$name) -> bool {
+                true
+            }
+            #[doc(hidden)]
+            #[no_coverage]
             fn validate_value(&self, _value: &$name) -> Option<Self::Cache> {
                 Some(())
             }
@@ -173,6 +179,13 @@ macro_rules! impl_int_mutator {
             fn default_mutation_step(&self, _value: &$name, _cache: &Self::Cache) -> Self::MutationStep {
                 INITIAL_MUTATION_STEP
             }
+
+            #[doc(hidden)]
+            #[no_coverage]
+            fn global_search_space_complexity(&self) -> f64 {
+                <$name>::BITS as f64
+            }
+
             /// The maximum complexity of an input of this type
             #[doc(hidden)]
             #[no_coverage]
@@ -217,6 +230,7 @@ macro_rules! impl_int_mutator {
                 value: &mut $name,
                 _cache: &mut Self::Cache,
                 step: &mut Self::MutationStep,
+                _subvalue_provider: &dyn crate::SubValueProvider,
                 max_cplx: f64,
             ) -> Option<(Self::UnmutateToken, f64)> {
                 if max_cplx < self.min_complexity() {
@@ -260,24 +274,14 @@ macro_rules! impl_int_mutator {
                 *value = t;
             }
 
-            #[doc(hidden)]
-            type RecursingPartIndex = ();
             #[doc(hidden)]
             #[no_coverage]
-            fn default_recursing_part_index(&self, _value: &$name, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
-            #[doc(hidden)]
-            #[no_coverage]
-            fn recursing_part<'a, V, N>(
+            fn visit_subvalues<'a>(
                 &self,
-                _parent: &N,
                 _value: &'a $name,
-                _index: &mut Self::RecursingPartIndex,
-            ) -> Option<&'a V>
-            where
-                V: Clone + 'static,
-                N: Mutator<V> + 'static,
-            {
-                None
+                _cache: &'a Self::Cache,
+                _visit: &mut dyn FnMut(&'a dyn Any, f64),
+            ) {
             }
         }
 
