@@ -1,10 +1,10 @@
+use std::any::Any;
+use std::ops::{Bound, RangeBounds};
+
 use crate::mutators::integer::{
     binary_search_arbitrary_u16, binary_search_arbitrary_u32, binary_search_arbitrary_u64, binary_search_arbitrary_u8,
 };
 use crate::Mutator;
-use std::ops::Bound;
-use std::ops::RangeBounds;
-
 const INITIAL_MUTATION_STEP: u64 = 0;
 
 macro_rules! impl_int_mutator_constrained {
@@ -12,6 +12,7 @@ macro_rules! impl_int_mutator_constrained {
         pub struct $name_mutator {
             start_range: $name,
             len_range: $name_unsigned,
+            search_space_complexity: f64,
             rng: fastrand::Rng,
         }
         impl $name_mutator {
@@ -41,9 +42,11 @@ macro_rules! impl_int_mutator_constrained {
                         range.end_bound()
                     )
                 }
+                let length = end.wrapping_sub(start);
                 Self {
                     start_range: start,
                     len_range: end.wrapping_sub(start) as $name_unsigned,
+                    search_space_complexity: super::size_to_cplxity(length as usize),
                     rng: fastrand::Rng::default(),
                 }
             }
@@ -64,11 +67,19 @@ macro_rules! impl_int_mutator_constrained {
             fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
                 0
             }
-
             #[doc(hidden)]
             #[no_coverage]
-            fn validate_value(&self, _value: &$name) -> Option<Self::Cache> {
-                Some(())
+            fn is_valid(&self, value: &$name) -> bool {
+                (self.start_range..=self.start_range.wrapping_add(self.len_range as _)).contains(value)
+            }
+            #[doc(hidden)]
+            #[no_coverage]
+            fn validate_value(&self, value: &$name) -> Option<Self::Cache> {
+                if (self.start_range..=self.start_range.wrapping_add(self.len_range as _)).contains(value) {
+                    Some(())
+                } else {
+                    None
+                }
             }
 
             #[doc(hidden)]
@@ -77,6 +88,12 @@ macro_rules! impl_int_mutator_constrained {
                 INITIAL_MUTATION_STEP
             }
 
+            #[doc(hidden)]
+            #[no_coverage]
+            fn global_search_space_complexity(&self) -> f64 {
+                self.search_space_complexity
+            }
+
             #[doc(hidden)]
             #[no_coverage]
             fn max_complexity(&self) -> f64 {
@@ -129,6 +146,7 @@ macro_rules! impl_int_mutator_constrained {
                 value: &mut $name,
                 _cache: &mut Self::Cache,
                 step: &mut Self::MutationStep,
+                _subvalue_provider: &dyn crate::SubValueProvider,
                 max_cplx: f64,
             ) -> Option<(Self::UnmutateToken, f64)> {
                 if max_cplx < self.min_complexity() {
@@ -170,24 +188,14 @@ macro_rules! impl_int_mutator_constrained {
                 *value = t;
             }
 
-            #[doc(hidden)]
-            type RecursingPartIndex = ();
             #[doc(hidden)]
             #[no_coverage]
-            fn default_recursing_part_index(&self, _value: &$name, _cache: &Self::Cache) -> Self::RecursingPartIndex {}
-            #[doc(hidden)]
-            #[no_coverage]
-            fn recursing_part<'a, V, N>(
+            fn visit_subvalues<'a>(
                 &self,
-                _parent: &N,
                 _value: &'a $name,
-                _index: &mut Self::RecursingPartIndex,
-            ) -> Option<&'a V>
-            where
-                V: Clone + 'static,
-                N: Mutator<V>,
-            {
-                None
+                _cache: &'a Self::Cache,
+                _visit: &mut dyn FnMut(&'a dyn Any, f64),
+            ) {
             }
         }
     };
@@ -201,3 +209,16 @@ impl_int_mutator_constrained!(i8, u8, I8WithinRangeMutator, binary_search_arbitr
 impl_int_mutator_constrained!(i16, u16, I16WithinRangeMutator, binary_search_arbitrary_u16);
 impl_int_mutator_constrained!(i32, u32, I32WithinRangeMutator, binary_search_arbitrary_u32);
 impl_int_mutator_constrained!(i64, u64, I64WithinRangeMutator, binary_search_arbitrary_u64);
+
+#[cfg(test)]
+mod tests {
+    use super::U8WithinRangeMutator;
+    use crate::Mutator;
+
+    #[test]
+    fn test_int_constrained() {
+        let m = U8WithinRangeMutator::new(1..2);
+        assert!(m.is_valid(&1));
+        assert!(!m.is_valid(&2));
+    }
+}
