@@ -1,14 +1,14 @@
 extern crate self as fuzzcheck;
 
-use crate::mutators::map::MapMutator;
-use crate::mutators::tuples::TupleMutatorWrapper;
-use crate::mutators::tuples::{Tuple2, Tuple2Mutator};
-use crate::mutators::wrapper::Wrapper;
-use crate::DefaultMutator;
-use crate::Mutator;
-use fuzzcheck_mutators_derive::make_mutator;
 use std::ops::{Bound, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo, RangeToInclusive};
 
+use fuzzcheck_mutators_derive::make_mutator;
+
+use crate::mutators::map::MapMutator;
+use crate::mutators::tuples::{Tuple2, Tuple2Mutator, TupleMutatorWrapper};
+use crate::mutators::Wrapper;
+use crate::{DefaultMutator, Mutator};
+
 make_mutator! {
     name: RangeMutator,
     default: true,
@@ -66,9 +66,10 @@ fn range_inclusive_from_tuple<T: Clone>(t: &(T, T)) -> RangeInclusive<T> {
 fn tuple_from_range_inclusive<T: Clone>(r: &RangeInclusive<T>) -> Option<(T, T)> {
     Some((r.start().clone(), r.end().clone()))
 }
+
 #[no_coverage]
-fn complexity<T: Clone>(_t: &RangeInclusive<T>, cplx: f64) -> f64 {
-    cplx
+fn range_cplx<T>(_r: &RangeInclusive<T>, orig_cplx: f64) -> f64 {
+    orig_cplx
 }
 
 pub type RangeInclusiveMutator<T, M> = Wrapper<
@@ -93,7 +94,7 @@ where
             TupleMutatorWrapper::new(Tuple2Mutator::new(m.clone(), m)),
             tuple_from_range_inclusive,
             range_inclusive_from_tuple,
-            complexity,
+            range_cplx,
         ))
     }
 }
