@@ -86,9 +86,11 @@
 //! None of it is *strictly* necessary since I could always write a brand new mutator for each type from scratch instead
 //! of trying to reuse mutators. But it would be a much larger amount of work, would probably increase compile times, and
 //! it would be more difficult to refactor and keep the implementations correct.
-use crate::Mutator;
+use std::any::Any;
 use std::marker::PhantomData;
 
+use crate::Mutator;
+
 /// A trait which essentially holds the types of a destructured tuple or structure.
 ///
 /// Read the [module documentation](crate::mutators::tuples) for more information about it.
@@ -134,10 +136,14 @@ where
 
     fn complexity<'a>(&self, value: TupleKind::Ref<'a>, cache: &'a Self::Cache) -> f64;
 
+    fn is_valid<'a>(&self, value: TupleKind::Ref<'a>) -> bool;
+
     fn validate_value<'a>(&self, value: TupleKind::Ref<'a>) -> Option<Self::Cache>;
 
     fn default_mutation_step<'a>(&self, value: TupleKind::Ref<'a>, cache: &'a Self::Cache) -> Self::MutationStep;
 
+    fn global_search_space_complexity(&self) -> f64;
+
     fn max_complexity(&self) -> f64;
 
     fn min_complexity(&self) -> f64;
@@ -151,6 +157,7 @@ where
         value: TupleKind::Mut<'a>,
         cache: &'a mut Self::Cache,
         step: &'a mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)>;
 
@@ -163,21 +170,12 @@ where
 
     fn unmutate<'a>(&self, value: TupleKind::Mut<'a>, cache: &'a mut Self::Cache, t: Self::UnmutateToken);
 
-    type RecursingPartIndex: Clone;
-    fn default_recursing_part_index<'a>(
+    fn visit_subvalues<'a>(
         &self,
         value: TupleKind::Ref<'a>,
-        cache: &Self::Cache,
-    ) -> Self::RecursingPartIndex;
-    fn recursing_part<'a, V, N>(
-        &self,
-        parent: &N,
-        value: TupleKind::Ref<'a>,
-        index: &mut Self::RecursingPartIndex,
-    ) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>;
+        cache: &'a Self::Cache,
+        visit: &mut dyn FnMut(&'a dyn Any, f64),
+    );
 }
 
 /// A wrapper that transforms a [`TupleMutator`] into a [`Mutator`] of values [with a tuple structure](TupleStructure).
@@ -238,8 +236,8 @@ where
 
     #[doc(hidden)]
     #[no_coverage]
-    fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
-        self.mutator.complexity(value.get_ref(), cache)
+    fn is_valid(&self, value: &T) -> bool {
+        self.mutator.is_valid(value.get_ref())
     }
 
     #[doc(hidden)]
@@ -253,6 +251,12 @@ where
         self.mutator.default_mutation_step(value.get_ref(), cache)
     }
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn global_search_space_complexity(&self) -> f64 {
+        self.mutator.global_search_space_complexity()
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn max_complexity(&self) -> f64 {
@@ -265,6 +269,12 @@ where
         self.mutator.min_complexity()
     }
 
+    #[doc(hidden)]
+    #[no_coverage]
+    fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
+        self.mutator.complexity(value.get_ref(), cache)
+    }
+
     #[doc(hidden)]
     #[no_coverage]
     fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
@@ -284,9 +294,11 @@ where
         value: &mut T,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn crate::SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)> {
-        self.mutator.ordered_mutate(value.get_mut(), cache, step, max_cplx)
+        self.mutator
+            .ordered_mutate(value.get_mut(), cache, step, subvalue_provider, max_cplx)
     }
 
     #[doc(hidden)]
@@ -301,32 +313,20 @@ where
         self.mutator.unmutate(value.get_mut(), cache, t)
     }
 
-    #[doc(hidden)]
-    type RecursingPartIndex = M::RecursingPartIndex;
-
     #[doc(hidden)]
     #[no_coverage]
-    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
-        self.mutator.default_recursing_part_index(value.get_ref(), cache)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>,
-    {
-        self.mutator.recursing_part::<V, N>(parent, value.get_ref(), index)
+    fn visit_subvalues<'a>(&self, value: &'a T, cache: &'a Self::Cache, visit: &mut dyn FnMut(&'a dyn Any, f64)) {
+        self.mutator.visit_subvalues(value.get_ref(), cache, visit)
     }
 }
 
 pub use tuple0::{Tuple0, Tuple0Mutator};
 mod tuple0 {
+
+    use std::any::Any;
+
     use super::TupleMutator;
-    use crate::mutators::tuples::RefTypes;
-    use crate::mutators::tuples::TupleStructure;
-    use crate::Mutator;
+    use crate::mutators::tuples::{RefTypes, TupleStructure};
 
     /// A marker type implementing [`RefTypes`] indicating that a type is equivalent to the unit type `()`
     pub struct Tuple0;
@@ -334,13 +334,15 @@ mod tuple0 {
         type Owned = ();
         type Ref<'a> = ();
         type Mut<'a> = ();
+        #[no_coverage]
         fn get_ref_from_mut<'a>(_v: &'a Self::Mut<'a>) -> Self::Ref<'a> {}
     }
     impl TupleStructure<Tuple0> for () {
+        #[no_coverage]
         fn get_ref(&self) -> <Tuple0 as RefTypes>::Ref<'_> {}
-
+        #[no_coverage]
         fn get_mut(&mut self) -> <Tuple0 as RefTypes>::Mut<'_> {}
-
+        #[no_coverage]
         fn new(_t: <Tuple0 as RefTypes>::Owned) -> Self {}
     }
     /// A `TupleMutator` for types equivalent to the unit type `()`
@@ -367,17 +369,30 @@ mod tuple0 {
             0.0
         }
 
+        #[doc(hidden)]
+        #[no_coverage]
+        fn is_valid<'a>(&self, _value: <Tuple0 as RefTypes>::Ref<'a>) -> bool {
+            true
+        }
+
         #[doc(hidden)]
         #[no_coverage]
         fn validate_value(&self, _value: ()) -> Option<Self::Cache> {
             Some(())
         }
+
         #[doc(hidden)]
         #[no_coverage]
         fn default_mutation_step<'a>(&self, _value: (), _cache: &'a Self::Cache) -> Self::MutationStep {
             false
         }
 
+        #[doc(hidden)]
+        #[no_coverage]
+        fn global_search_space_complexity(&self) -> f64 {
+            0.0
+        }
+
         #[doc(hidden)]
         #[no_coverage]
         fn max_complexity(&self) -> f64 {
@@ -414,6 +429,7 @@ mod tuple0 {
             _value: (),
             _cache: &mut Self::Cache,
             step: &mut Self::MutationStep,
+            _subvalue_provider: &dyn crate::SubValueProvider,
             _max_cplx: f64,
         ) -> Option<(Self::UnmutateToken, f64)> {
             if !*step {
@@ -434,34 +450,19 @@ mod tuple0 {
         #[no_coverage]
         fn unmutate(&self, _value: (), _cache: &mut Self::Cache, _t: Self::UnmutateToken) {}
 
-        #[doc(hidden)]
-        type RecursingPartIndex = ();
-
         #[doc(hidden)]
         #[no_coverage]
-        fn default_recursing_part_index(&self, _value: (), _cache: &Self::Cache) -> Self::RecursingPartIndex {}
-
-        #[doc(hidden)]
-        #[no_coverage]
-        fn recursing_part<'a, V, N>(
-            &self,
-            _parent: &N,
-            _value: (),
-            _index: &mut Self::RecursingPartIndex,
-        ) -> Option<&'a V>
-        where
-            V: Clone + 'static,
-            N: Mutator<V>,
-        {
-            None
-        }
+        fn visit_subvalues<'a>(&self, _value: (), _cache: &'a Self::Cache, _visit: &mut dyn FnMut(&'a dyn Any, f64)) {}
     }
 }
 
 pub use tuple1::{Tuple1, Tuple1Mutator};
 mod tuple1 {
+    use std::any::Any;
+
     use super::{TupleMutator, TupleMutatorWrapper};
     use crate::mutators::tuples::RefTypes;
+    use crate::mutators::{CrossoverStep, CROSSOVER_RATE};
 
     #[doc = "A marker type implementing [`RefTypes`](crate::mutators::tuples::RefTypes) indicating that a type has the [structure](crate::mutators::tuples::TupleStructure) of a 1-tuple."]
     pub struct Tuple1<T0: 'static> {
@@ -490,15 +491,29 @@ mod tuple1 {
             t
         }
     }
+    #[derive(Clone)]
+    pub struct MutationStep<T, MS> {
+        crossover_step: CrossoverStep<T>,
+        inner: MS,
+    }
+    pub enum UnmutateTuple1Token<T, U> {
+        Replace(T),
+        Inner(U),
+    }
+
     #[doc = " A `TupleMutator` for types that have a 1-tuple structure"]
     #[derive(::std::default::Default)]
     pub struct Tuple1Mutator<M0> {
         mutator_0: M0,
+        rng: fastrand::Rng,
     }
     impl<M0> Tuple1Mutator<M0> {
         #[no_coverage]
         pub fn new(mutator_0: M0) -> Self {
-            Self { mutator_0 }
+            Self {
+                mutator_0,
+                rng: fastrand::Rng::new(),
+            }
         }
     }
 
@@ -512,19 +527,54 @@ mod tuple1 {
         #[doc(hidden)]
         type Cache = <M0 as crate::Mutator<T0>>::Cache;
         #[doc(hidden)]
-        type MutationStep = <M0 as crate::Mutator<T0>>::MutationStep;
-        #[doc(hidden)]
-        type RecursingPartIndex = <M0 as crate::Mutator<T0>>::RecursingPartIndex;
+        type MutationStep = MutationStep<T0, <M0 as crate::Mutator<T0>>::MutationStep>;
         #[doc(hidden)]
         type ArbitraryStep = <M0 as crate::Mutator<T0>>::ArbitraryStep;
         #[doc(hidden)]
-        type UnmutateToken = <M0 as crate::Mutator<T0>>::UnmutateToken;
+        type UnmutateToken = UnmutateTuple1Token<T0, <M0 as crate::Mutator<T0>>::UnmutateToken>;
+
         #[doc(hidden)]
         #[no_coverage]
         fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
             self.mutator_0.default_arbitrary_step()
         }
 
+        #[doc(hidden)]
+        #[no_coverage]
+        fn complexity<'a>(&self, value: <Tuple1<T0> as RefTypes>::Ref<'a>, cache: &'a Self::Cache) -> f64 {
+            self.mutator_0.complexity(value.0, cache)
+        }
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn is_valid<'a>(&self, value: <Tuple1<T0> as RefTypes>::Ref<'a>) -> bool {
+            self.mutator_0.is_valid(value.0)
+        }
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn validate_value<'a>(&self, value: <Tuple1<T0> as RefTypes>::Ref<'a>) -> Option<Self::Cache> {
+            self.mutator_0.validate_value(value.0)
+        }
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn default_mutation_step<'a>(
+            &self,
+            value: <Tuple1<T0> as RefTypes>::Ref<'a>,
+            cache: &'a Self::Cache,
+        ) -> Self::MutationStep {
+            MutationStep {
+                crossover_step: CrossoverStep::default(),
+                inner: self.mutator_0.default_mutation_step(value.0, cache),
+            }
+        }
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn global_search_space_complexity(&self) -> f64 {
+            self.mutator_0.global_search_space_complexity()
+        }
         #[doc(hidden)]
         #[no_coverage]
         fn max_complexity(&self) -> f64 {
@@ -537,29 +587,11 @@ mod tuple1 {
         }
         #[doc(hidden)]
         #[no_coverage]
-        fn complexity<'a>(&self, value: <Tuple1<T0> as RefTypes>::Ref<'a>, cache: &'a Self::Cache) -> f64 {
-            self.mutator_0.complexity(value.0, cache)
-        }
-        #[doc(hidden)]
-        #[no_coverage]
-        fn validate_value<'a>(&self, value: <Tuple1<T0> as RefTypes>::Ref<'a>) -> Option<Self::Cache> {
-            self.mutator_0.validate_value(value.0)
-        }
-        #[doc(hidden)]
-        #[no_coverage]
-        fn default_mutation_step<'a>(
-            &self,
-            value: <Tuple1<T0> as RefTypes>::Ref<'a>,
-            cache: &'a Self::Cache,
-        ) -> Self::MutationStep {
-            self.mutator_0.default_mutation_step(value.0, cache)
-        }
-        #[doc(hidden)]
-        #[no_coverage]
         fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
-            self.mutator_0
-                .ordered_arbitrary(step, max_cplx)
-                .map(|(value, cplx)| (T::new((value,)), cplx))
+            self.mutator_0.ordered_arbitrary(step, max_cplx).map(
+                #[no_coverage]
+                |(value, cplx)| (T::new((value,)), cplx),
+            )
         }
         #[doc(hidden)]
         #[no_coverage]
@@ -574,9 +606,26 @@ mod tuple1 {
             value: <Tuple1<T0> as RefTypes>::Mut<'a>,
             cache: &'a mut Self::Cache,
             step: &'a mut Self::MutationStep,
+            subvalue_provider: &dyn crate::SubValueProvider,
             max_cplx: f64,
         ) -> Option<(Self::UnmutateToken, f64)> {
-            self.mutator_0.ordered_mutate(value.0, cache, step, max_cplx)
+            if self.rng.u8(..CROSSOVER_RATE) == 0 {
+                if let Some((subvalue, subcplx)) = step.crossover_step.get_next_subvalue(subvalue_provider, max_cplx) {
+                    if self.mutator_0.is_valid(subvalue) {
+                        let mut replacer = subvalue.clone();
+                        std::mem::swap(value.0, &mut replacer);
+                        return Some((UnmutateTuple1Token::Replace(replacer), subcplx));
+                    }
+                }
+            }
+            if let Some((token, cplx)) =
+                self.mutator_0
+                    .ordered_mutate(value.0, cache, &mut step.inner, subvalue_provider, max_cplx)
+            {
+                Some((UnmutateTuple1Token::Inner(token), cplx))
+            } else {
+                None
+            }
         }
         #[doc(hidden)]
         #[no_coverage]
@@ -586,8 +635,10 @@ mod tuple1 {
             cache: &'a mut Self::Cache,
             max_cplx: f64,
         ) -> (Self::UnmutateToken, f64) {
-            self.mutator_0.random_mutate(value.0, cache, max_cplx)
+            let (token, cplx) = self.mutator_0.random_mutate(value.0, cache, max_cplx);
+            (UnmutateTuple1Token::Inner(token), cplx)
         }
+
         #[doc(hidden)]
         #[no_coverage]
         fn unmutate<'a>(
@@ -596,30 +647,25 @@ mod tuple1 {
             cache: &'a mut Self::Cache,
             t: Self::UnmutateToken,
         ) {
-            self.mutator_0.unmutate(value.0, cache, t);
+            match t {
+                UnmutateTuple1Token::Replace(x) => {
+                    *value.0 = x;
+                }
+                UnmutateTuple1Token::Inner(t) => self.mutator_0.unmutate(value.0, cache, t),
+            }
         }
+
         #[doc(hidden)]
         #[no_coverage]
-        fn default_recursing_part_index<'a>(
+        fn visit_subvalues<'a>(
             &self,
             value: <Tuple1<T0> as RefTypes>::Ref<'a>,
             cache: &'a Self::Cache,
-        ) -> Self::RecursingPartIndex {
-            self.mutator_0.default_recursing_part_index(value.0, cache)
-        }
-        #[doc(hidden)]
-        #[no_coverage]
-        fn recursing_part<'a, ___V, ___N>(
-            &self,
-            parent: &___N,
-            value: <Tuple1<T0> as RefTypes>::Ref<'a>,
-            index: &mut Self::RecursingPartIndex,
-        ) -> Option<&'a ___V>
-        where
-            ___V: ::std::clone::Clone + 'static,
-            ___N: crate::Mutator<___V>,
-        {
-            self.mutator_0.recursing_part::<___V, ___N>(parent, value.0, index)
+            visit: &mut dyn FnMut(&'a dyn Any, f64),
+        ) {
+            let cplx = self.mutator_0.complexity(value.0, cache);
+            visit(value.0, cplx);
+            self.mutator_0.visit_subvalues(value.0, cache, visit);
         }
     }
     impl<T0> crate::mutators::DefaultMutator for (T0,)
