@@ -1,13 +1,10 @@
-use super::arbitrary;
-use super::insert_element;
-use super::insert_many_elements;
-use super::mutate_element;
-use super::only_choose_length;
-use super::remove;
-use super::remove_and_insert_element;
-use super::swap_elements;
-use super::VecMutator;
-use crate::mutators::mutations::{Mutation, RevertMutation};
+use super::crossover_insert_slice::CrossoverInsertSlice;
+use super::crossover_replace_element::CrossoverReplaceElement;
+use super::{
+    arbitrary, copy_element, crossover_insert_slice, crossover_replace_element, insert_element, insert_many_elements,
+    mutate_element, only_choose_length, remove, remove_and_insert_element, swap_elements, VecMutator,
+};
+use crate::mutators::mutations::{Mutation, NoMutation, RevertMutation};
 use crate::mutators::vose_alias::VoseAlias;
 use crate::Mutator;
 
@@ -74,6 +71,7 @@ macro_rules! impl_vec_mutation {
             T: Clone + 'static,
             M: Mutator<T>,
         {
+            #[no_coverage]
             fn clone(&self) -> Self {
                 match self {
                     $(
@@ -89,6 +87,7 @@ macro_rules! impl_vec_mutation {
             M: Mutator<T>,
             VectorMutationInnerStep<T, M>: Clone
         {
+            #[no_coverage]
             fn clone(&self) -> Self {
                 Self {
                     inner_steps: self.inner_steps.clone(),
@@ -103,6 +102,8 @@ macro_rules! impl_vec_mutation {
             T: Clone + 'static,
             M: Mutator<T>,
         {
+            #[allow(unreachable_code)]
+            #[no_coverage]
             fn clone(&self) -> Self {
                 match self {
                     $(
@@ -118,6 +119,7 @@ macro_rules! impl_vec_mutation {
             M: Mutator<T>,
             VectorMutationInnerRandomStep<T, M>: Clone
         {
+            #[no_coverage]
             fn clone(&self) -> Self {
                 Self {
                     inner_steps: self.inner_steps.clone(),
@@ -131,6 +133,7 @@ macro_rules! impl_vec_mutation {
             T: Clone + 'static,
             M: Mutator<T>,
         {
+            #[no_coverage]
             fn revert(
                 self,
                 mutator: &VecMutator<T, M>,
@@ -154,12 +157,12 @@ macro_rules! impl_vec_mutation {
             type Step = VectorMutationStep<T, M>;
             type Concrete<'a> = ConcreteVectorMutation<'a, T, M>;
             type Revert = RevertVectorMutation<T, M>;
-
+            #[no_coverage]
             fn default_random_step(&self, mutator: &VecMutator<T, M>, value: &Vec<T>) -> Option<Self::RandomStep> {
                 let inner_steps_and_weights: Vec<(_, f64)> = self
                     .mutations
                     .iter()
-                    .filter_map(|mutation| {
+                    .filter_map(#[no_coverage] |mutation| {
                         match &mutation.mutation {
                             $(
                                 InnerVectorMutation::$i(r) => r
@@ -167,20 +170,20 @@ macro_rules! impl_vec_mutation {
                                     .map(VectorMutationInnerRandomStep::$i)
                             ),*
                         }
-                        .map(|inner| (inner, mutation.random_weight))
+                        .map(#[no_coverage] |inner| (inner, mutation.random_weight))
                     })
                     .collect::<Vec<_>>();
 
                 if inner_steps_and_weights.is_empty() {
                     return None;
                 }
-                let weights = inner_steps_and_weights.iter().map(|x| x.1).collect();
+                let weights = inner_steps_and_weights.iter().map(#[no_coverage] |x| x.1).collect();
                 let sampling = VoseAlias::new(weights);
-                let inner_steps = inner_steps_and_weights.into_iter().map(|x| x.0).collect();
+                let inner_steps = inner_steps_and_weights.into_iter().map(#[no_coverage] |x| x.0).collect();
 
                 Some(VectorMutationRandomStep { inner_steps, sampling })
             }
-
+            #[no_coverage]
             fn random<'a>(
                 mutator: &VecMutator<T, M>,
                 value: &Vec<T>,
@@ -197,7 +200,7 @@ macro_rules! impl_vec_mutation {
                     ),*
                 }
             }
-
+            #[no_coverage]
             fn default_step(
                 &self,
                 mutator: &VecMutator<T, M>,
@@ -207,7 +210,7 @@ macro_rules! impl_vec_mutation {
                 let inner_steps_and_weights: Vec<(VectorMutationInnerStep<_, _>, f64)> = self
                     .mutations
                     .iter()
-                    .filter_map(|mutation| {
+                    .filter_map(#[no_coverage] |mutation| {
                         match &mutation.mutation {
                             $(
                                 InnerVectorMutation::$i(r) =>
@@ -215,7 +218,7 @@ macro_rules! impl_vec_mutation {
                                     .map(VectorMutationInnerStep::$i)
                             ),*
                         }
-                        .map(|inner| (inner, mutation.ordered_weight))
+                        .map(#[no_coverage] |inner| (inner, mutation.ordered_weight))
                     })
                     .collect::<Vec<_>>();
 
@@ -239,12 +242,13 @@ macro_rules! impl_vec_mutation {
                     sampling,
                 })
             }
-
+            #[no_coverage]
             fn from_step<'a>(
                 mutator: &VecMutator<T, M>,
                 value: &Vec<T>,
                 cache: &<VecMutator<T, M> as Mutator<Vec<T>>>::Cache,
                 step: &'a mut Self::Step,
+                subvalue_provider: &dyn crate::SubValueProvider,
                 max_cplx: f64,
             ) -> Option<Self::Concrete<'a>> {
                 if step.inner_steps.is_empty() {
@@ -259,7 +263,7 @@ macro_rules! impl_vec_mutation {
                         match inner_step {
                             $(
                                 VectorMutationInnerStep::$i(step) =>
-                                    <$t>::from_step(mutator, value, cache, step, max_cplx)
+                                    <$t>::from_step(mutator, value, cache, step, subvalue_provider, max_cplx)
                                     .map(ConcreteVectorMutation::$i)
                             ),*
                         };
@@ -276,21 +280,22 @@ macro_rules! impl_vec_mutation {
                     None
                 } else {
                     step.sampling = VoseAlias::new(step.weights.clone());
-                    Self::from_step(mutator, value, cache, step, max_cplx)
+                    Self::from_step(mutator, value, cache, step, subvalue_provider, max_cplx)
                 }
             }
-
+            #[no_coverage]
             fn apply<'a>(
                 mutation: Self::Concrete<'a>,
                 mutator: &VecMutator<T, M>,
                 value: &mut Vec<T>,
                 cache: &mut <VecMutator<T, M> as Mutator<Vec<T>>>::Cache,
+                subvalue_provider: &dyn crate::SubValueProvider,
                 max_cplx: f64,
             ) -> (Self::Revert, f64) {
                 match mutation {
                     $(
                         ConcreteVectorMutation::$i(mutation) => {
-                            let (revert, cplx) =  <$t>::apply(mutation, mutator, value, cache, max_cplx);
+                            let (revert, cplx) =  <$t>::apply(mutation, mutator, value, cache, subvalue_provider, max_cplx);
                             (RevertVectorMutation::$i(revert), cplx)
                         }
                     )*
@@ -301,6 +306,8 @@ macro_rules! impl_vec_mutation {
 }
 
 impl_vec_mutation! {
+    (NoMutation, NoMutation),
+    (CopyElement, copy_element::CopyElement),
     (Remove, remove::Remove),
     (MutateElement, mutate_element::MutateElement),
     (InsertElement, insert_element::InsertElement),
@@ -308,16 +315,71 @@ impl_vec_mutation! {
     (InsertManyElements, insert_many_elements::InsertManyElements),
     (RemoveAndInsertElement, remove_and_insert_element::RemoveAndInsertElement),
     (OnlyChooseLength, only_choose_length::OnlyChooseLength),
-    (Arbitrary, arbitrary::Arbitrary)
+    (Arbitrary, arbitrary::Arbitrary),
+    (CrossoverReplaceElement, crossover_replace_element::CrossoverReplaceElement),
+    (CrossoverInsertSlice, crossover_insert_slice::CrossoverInsertSlice)
+}
+
+impl<'a, T, M> std::fmt::Debug for ConcreteVectorMutation<'a, T, M>
+where
+    T: Clone + 'static,
+    M: Mutator<T>,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            ConcreteVectorMutation::NoMutation(_) => {
+                write!(f, "NoMutation")
+            }
+            ConcreteVectorMutation::CopyElement(_) => {
+                write!(f, "CopyElement")
+            }
+            ConcreteVectorMutation::Remove(_) => {
+                write!(f, "Remove")
+            }
+            ConcreteVectorMutation::MutateElement(_) => {
+                write!(f, "MutateElement")
+            }
+            ConcreteVectorMutation::InsertElement(_) => {
+                write!(f, "InsertElement")
+            }
+            ConcreteVectorMutation::SwapElements(_) => {
+                write!(f, "SwapElements")
+            }
+            ConcreteVectorMutation::InsertManyElements(_) => {
+                write!(f, "InsertManyElements")
+            }
+            ConcreteVectorMutation::RemoveAndInsertElement(_) => {
+                write!(f, "RemoveAndInsertElement")
+            }
+            ConcreteVectorMutation::OnlyChooseLength(_) => {
+                write!(f, "OnlyChooseLength")
+            }
+            ConcreteVectorMutation::Arbitrary(_) => {
+                write!(f, "Arbitrary")
+            }
+            ConcreteVectorMutation::CrossoverReplaceElement(_) => {
+                write!(f, "CrossoverReplaceElement")
+            }
+            ConcreteVectorMutation::CrossoverInsertSlice(_) => {
+                write!(f, "CrossoverInsertSlice")
+            }
+        }
+    }
 }
 
 // ====== Default Vector Mutations =====
 
 impl Default for VectorMutation {
+    #[no_coverage]
     fn default() -> Self {
         // use the same standard for all of them
         Self {
             mutations: vec![
+                WeightedMutation {
+                    mutation: InnerVectorMutation::CopyElement(copy_element::CopyElement),
+                    random_weight: 50.,
+                    ordered_weight: 500.,
+                },
                 WeightedMutation {
                     mutation: InnerVectorMutation::OnlyChooseLength(only_choose_length::OnlyChooseLength),
                     random_weight: 1., // doesn't matter, it's the only mutation when relevant!
@@ -403,6 +465,16 @@ impl Default for VectorMutation {
                     random_weight: 8.,
                     ordered_weight: 4.,
                 },
+                WeightedMutation {
+                    mutation: InnerVectorMutation::CrossoverReplaceElement(CrossoverReplaceElement),
+                    random_weight: 0.,
+                    ordered_weight: 100.,
+                },
+                WeightedMutation {
+                    mutation: InnerVectorMutation::CrossoverInsertSlice(CrossoverInsertSlice),
+                    random_weight: 0.,
+                    ordered_weight: 50.,
+                },
                 // WeightedMutation {
                 //     mutation: InnerVectorMutation::InsertManyElements(insert_many_elements::InsertManyElements {
                 //         nbr_added_elements: 4,
