@@ -1,20 +1,13 @@
-use std::{
-    cmp::Ordering,
-    fmt::{self, Display, Formatter},
-    path::PathBuf,
-};
+use std::cmp::Ordering;
+use std::fmt::{self, Display, Formatter};
+use std::path::PathBuf;
 
-use crate::traits::Stats;
-use crate::{bitset::FixedBitSet, traits::SaveToStatsFolder};
+use crate::bitset::FixedBitSet;
+use crate::fenwick_tree::FenwickTree;
+use crate::traits::{CorpusDelta, Pool, SaveToStatsFolder, Stats};
+use crate::{CSVField, CompatibleWithObservations, PoolStorageIndex, ToCSV};
 
-use crate::{
-    fenwick_tree::FenwickTree,
-    fuzzer::PoolStorageIndex,
-    traits::{CorpusDelta, Pool},
-    CSVField, ToCSV,
-};
-
-use super::compatible_with_iterator_sensor::CompatibleWithIteratorSensor;
+// use super::compatible_with_iterator_sensor::CompatibleWithIteratorSensor;
 
 #[derive(Clone)]
 struct Input {
@@ -91,43 +84,9 @@ pub struct ObservationState {
     nbr_new_counters: usize,
 }
 
-impl CompatibleWithIteratorSensor for MostNDiversePool {
-    type Observation = (usize, u64);
-    type ObservationState = ObservationState;
-
+impl MostNDiversePool {
     #[no_coverage]
-    fn start_observing(&mut self) -> Self::ObservationState {
-        ObservationState {
-            counters: FixedBitSet::with_capacity(self.nbr_counters + 1),
-            nbr_new_counters: 0,
-        }
-    }
-
-    #[no_coverage]
-    fn observe(&mut self, observation: &Self::Observation, _input_complexity: f64, state: &mut Self::ObservationState) {
-        let ObservationState {
-            counters,
-            nbr_new_counters: _,
-        } = state;
-        let (idx, _) = observation;
-        counters.insert(*idx);
-    }
-    #[no_coverage]
-    fn finish_observing(&mut self, state: &mut Self::ObservationState, _input_complexity: f64) {
-        let ObservationState {
-            counters,
-            nbr_new_counters,
-        } = state;
-        self.cache.clone_from(counters);
-        // let mut unique_counters = counters.clone();
-        self.cache.difference_with(&self.all_counters);
-
-        *nbr_new_counters = self.cache.count_ones();
-        self.cache.clear();
-    }
-
-    #[no_coverage]
-    fn is_interesting(&self, state: &Self::ObservationState, input_complexity: f64) -> bool {
+    fn is_interesting(&self, state: &ObservationState, input_complexity: f64) -> bool {
         let ObservationState {
             counters,
             nbr_new_counters,
@@ -164,13 +123,33 @@ impl CompatibleWithIteratorSensor for MostNDiversePool {
 
         false
     }
+}
+
+impl<O> CompatibleWithObservations<O> for MostNDiversePool
+where
+    for<'a> &'a O: IntoIterator<Item = &'a (usize, u64)>,
+{
     #[no_coverage]
-    fn add(
-        &mut self,
-        data: PoolStorageIndex,
-        input_complexity: f64,
-        state: Self::ObservationState,
-    ) -> Vec<CorpusDelta> {
+    fn process(&mut self, input_id: PoolStorageIndex, observations: &O, complexity: f64) -> Vec<CorpusDelta> {
+        let mut state = ObservationState {
+            counters: FixedBitSet::with_capacity(self.nbr_counters + 1),
+            nbr_new_counters: 0,
+        };
+
+        for (index, _counter) in observations.into_iter() {
+            state.counters.insert(*index);
+        }
+
+        self.cache.clone_from(&state.counters);
+        // let mut unique_counters = counters.clone();
+        self.cache.difference_with(&self.all_counters);
+
+        state.nbr_new_counters = self.cache.count_ones();
+        self.cache.clear();
+
+        if !self.is_interesting(&state, complexity) {
+            return vec![];
+        }
         let ObservationState {
             counters,
             nbr_new_counters,
@@ -186,8 +165,8 @@ impl CompatibleWithIteratorSensor for MostNDiversePool {
             nbr_unique_counters: nbr_new_counters,
             unique_counters,
             counters: counters.clone(),
-            pool_idx: data,
-            cplx: input_complexity,
+            pool_idx: input_id,
+            cplx: complexity,
         };
 
         if self.inputs.len() < self.max_len && nbr_new_counters > 0 {
@@ -205,7 +184,7 @@ impl CompatibleWithIteratorSensor for MostNDiversePool {
             |idx| &mut self.inputs[idx],
         ) {
             if (nbr_new_counters > worst_input.nbr_unique_counters)
-                || (nbr_new_counters == worst_input.nbr_unique_counters && worst_input.cplx > input_complexity)
+                || (nbr_new_counters == worst_input.nbr_unique_counters && worst_input.cplx > complexity)
             {
                 let worst_input_data = worst_input.pool_idx;
                 *worst_input = new_input;
@@ -220,14 +199,14 @@ impl CompatibleWithIteratorSensor for MostNDiversePool {
 
         let mut common_unique_counters = FixedBitSet::with_capacity(counters.len());
         for input in &mut self.inputs {
-            if nbr_new_counters > 0 || input.cplx > input_complexity {
+            if nbr_new_counters > 0 || input.cplx > complexity {
                 common_unique_counters.clone_from(&counters);
                 common_unique_counters.intersect_with(&input.unique_counters);
                 let common_uniq_counters = common_unique_counters.count_ones();
                 let nbr_new_counters = common_uniq_counters + nbr_new_counters;
 
                 if (nbr_new_counters > input.nbr_unique_counters)
-                    || (nbr_new_counters == input.nbr_unique_counters && input.cplx > input_complexity)
+                    || (nbr_new_counters == input.nbr_unique_counters && input.cplx > complexity)
                 {
                     let input_data = input.pool_idx;
                     *input = new_input;
@@ -284,7 +263,15 @@ impl MostNDiversePool {
                 #[no_coverage]
                 |x| x.0,
             );
-        self.fenwick_tree = FenwickTree::new(self.inputs.iter().map(|x| x.nbr_unique_counters as f64).collect());
+        self.fenwick_tree = FenwickTree::new(
+            self.inputs
+                .iter()
+                .map(
+                    #[no_coverage]
+                    |x| x.nbr_unique_counters as f64,
+                )
+                .collect(),
+        );
     }
 }
 
@@ -330,12 +317,16 @@ mod tests {
 
     #[no_coverage]
     fn run(pool: &mut MostNDiversePool, observations: Vec<usize>, cplx: f64) {
-        let mut obs_state = pool.start_observing();
-        for observation in &observations {
-            pool.observe(&(*observation, 1), cplx, &mut obs_state);
-        }
-        if pool.is_interesting(&obs_state, cplx) {
-            pool.add(PoolStorageIndex::mock(0), cplx, obs_state);
+        let observations = observations
+            .iter()
+            .map(
+                #[no_coverage]
+                |x| (*x, 1u64),
+            )
+            .collect::<Vec<_>>();
+        let corpus = pool.process(PoolStorageIndex::mock(0), &observations, cplx);
+
+        if !corpus.is_empty() {
             println!(
                 "input_count: {} worst_idx: {:?}, all_counters: {}",
                 pool.inputs.len(),
