@@ -6,18 +6,18 @@ use crate::traits::{SaveToStatsFolder, Sensor};
 ///
 /// In practice, it is used in conjunction with [`UnitPool`](crate::sensors_and_pools::UnitPool) to
 /// favour one particular test case throughout the whole fuzzing run. This is partly how the `minify`
-/// (test case minify) command is implemented.
+/// command is implemented.
 pub struct NoopSensor;
 
 impl Sensor for NoopSensor {
-    type ObservationHandler<'a> = ();
+    type Observations = ();
     #[no_coverage]
     fn start_recording(&mut self) {}
     #[no_coverage]
     fn stop_recording(&mut self) {}
 
     #[no_coverage]
-    fn iterate_over_observations(&mut self, _handler: Self::ObservationHandler<'_>) {}
+    fn get_observations(&mut self) {}
 }
 impl SaveToStatsFolder for NoopSensor {
     #[no_coverage]
