@@ -1,10 +1,11 @@
-use crate::fuzzer::PoolStorageIndex;
-use crate::traits::{CompatibleWithSensor, CorpusDelta, Pool, SaveToStatsFolder, Sensor, Stats};
-use crate::{CSVField, ToCSV};
-use nu_ansi_term::Color;
 use std::fmt::Display;
 use std::path::PathBuf;
 
+use nu_ansi_term::Color;
+
+use crate::traits::{CompatibleWithObservations, CorpusDelta, Pool, SaveToStatsFolder, Sensor, Stats};
+use crate::{CSVField, PoolStorageIndex, ToCSV};
+
 const NBR_ARTIFACTS_PER_ERROR_AND_CPLX: usize = 8;
 
 pub(crate) static mut TEST_FAILURE: Option<TestFailure> = None;
@@ -19,14 +20,13 @@ pub struct TestFailure {
 }
 
 /// A sensor that records test failures.
-///
-/// It is [compatible with](CompatibleWithSensor) [`TestFailurePool`].
 #[derive(Default)]
 pub struct TestFailureSensor {
     error: Option<TestFailure>,
 }
+
 impl Sensor for TestFailureSensor {
-    type ObservationHandler<'a> = &'a mut Option<TestFailure>;
+    type Observations = Option<TestFailure>;
 
     #[no_coverage]
     fn start_recording(&mut self) {
@@ -44,8 +44,8 @@ impl Sensor for TestFailureSensor {
     }
 
     #[no_coverage]
-    fn iterate_over_observations(&mut self, handler: Self::ObservationHandler<'_>) {
-        *handler = std::mem::take(&mut self.error);
+    fn get_observations(&mut self) -> Option<TestFailure> {
+        std::mem::take(&mut self.error)
     }
 }
 impl SaveToStatsFolder for TestFailureSensor {
@@ -94,8 +94,6 @@ struct TestFailureListForError {
 /// A pool that saves failing test cases.
 ///
 /// It categorizes the test cases by their failure information and sort them by complexity.
-///
-/// It is [compatible with](crate::CompatibleWithSensor) [`TestFailureSensor`]
 pub struct TestFailurePool {
     name: String,
     inputs: Vec<TestFailureList>,
@@ -142,20 +140,28 @@ impl Pool for TestFailurePool {
 impl SaveToStatsFolder for TestFailurePool {
     #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
-        vec![]
+        cfg_if::cfg_if! {
+            if #[cfg(feature = "serde_json_serializer")]
+            {
+                let path = PathBuf::new().join("test_failures.json");
+                let content = serde_json::to_string(&self.inputs.iter().map(|tf| (tf.error.id, tf.error.display.clone()) ).collect::<Vec<_>>()).unwrap();
+                vec![(path, content.into_bytes())]
+            } else {
+                vec![]
+            }
+        }
     }
 }
 
-impl CompatibleWithSensor<TestFailureSensor> for TestFailurePool {
+impl CompatibleWithObservations<Option<TestFailure>> for TestFailurePool {
     #[no_coverage]
     fn process(
         &mut self,
         input_idx: PoolStorageIndex,
-        sensor: &mut TestFailureSensor,
+        observations: &Option<TestFailure>,
         complexity: f64,
     ) -> Vec<CorpusDelta> {
-        let mut error = None;
-        sensor.iterate_over_observations(&mut error);
+        let error = observations;
 
         enum PositionOfNewInput {
             NewError,
@@ -199,7 +205,7 @@ impl CompatibleWithSensor<TestFailureSensor> for TestFailurePool {
                 match position {
                     PositionOfNewInput::NewError => {
                         self.inputs.push(TestFailureList {
-                            error,
+                            error: error.clone(),
                             inputs: vec![TestFailureListForError {
                                 cplx: complexity,
                                 inputs: vec![input_idx],
