@@ -1,111 +1,128 @@
-/*!
-The fuzzcheck_traits crate defines the `Mutator` and `Serializer` traits
-used by all fuzzcheck-related crates.
-*/
-
-use crate::fuzzer::PoolStorageIndex;
-use fuzzcheck_common::FuzzerEvent;
+use std::any::Any;
 use std::fmt::Display;
 use std::path::PathBuf;
 
+use fuzzcheck_common::FuzzerEvent;
+
+use crate::fuzzer::PoolStorageIndex;
+use crate::subvalue_provider::SubValueProvider;
+
 /**
- A [`Mutator`] is an object capable of generating/mutating a value for the purpose of
- fuzz-testing.
+A [`Mutator`] is an object capable of generating/mutating a value for the purpose of
+fuzz-testing.
 
- For example, a mutator could change the value
- `v1 = [1, 4, 2, 1]` to `v1' = [1, 5, 2, 1]`.
- The idea is that if `v1` is an “interesting” value to test, then `v1'` also
- has a high chance of being “interesting” to test.
+For example, a mutator could change the value
+`v1 = [1, 4, 2, 1]` to `v1' = [1, 5, 2, 1]`.
+The idea is that if `v1` is an “interesting” value to test, then `v1'` also
+has a high chance of being “interesting” to test.
 
- Fuzzcheck itself provides a few mutators for `std` types as well as procedural macros
- to generate mutators. See the [`mutators`](crate::mutators) module.
+Fuzzcheck itself provides a few mutators for `std` types as well as procedural macros
+to generate mutators. See the [`mutators`](crate::mutators) module.
 
- ## Complexity
+## Complexity
 
- A mutator is also responsible for keeping track of the
- [complexity](crate::Mutator::complexity) of a value. The complexity is,
- roughly speaking, how large the value is.
+A mutator is also responsible for keeping track of the
+[complexity](crate::Mutator::complexity) of a value. The complexity is,
+roughly speaking, how large the value is.
 
- If the complexity is `0.0`, then it means the mutator can only produce one value.
+For example, the complexity of a vector could be the sum of the complexities
+of its elements. So `vec![]` would have a complexity of `1.0` (what we chose as
+the base complexity of a vector) and `vec![76]` would have a complexity of
+`9.0`: `1.0` for the base complexity of the vector itself + `8.0` for the 8-bit
+integer “76”. There is no fixed rule for how to compute the complexity of a
+value. However, all mutators of a value of type MUST agree on what its
+complexity is within a fuzz-test. In other words, if we have the following
+mutator for the type `(u8, u8)`:
+```ignore
+struct MutatorTuple2<M1, M2> where M1: Mutator<u8>, M2: Mutator<u8> {
+   m1: M1, // responsible for mutating the first element
+   m2: M2  // responsible for mutating the second element
+}
+```
+then the submutators `M1` and `M2` must always give the same complexity
+for all values of type `u8`.
 
- For example, the complexity of a vector could be the sum of the complexities
- of its elements. So `vec![]` would have a complexity of `1.0` (what we chose as
- the base complexity of a vector) and `vec![76]` would have a complexity of
- `9.0`: `1.0` for the base complexity of the vector itself + `8.0` for the 8-bit
- integer “76”. But there is no fixed rule for how to compute the complexity of a
- value, and it is up to you to judge how “large” something is.
+## Global search space complexity
 
- ## [`Cache`](Mutator::Cache)
+The search space complexity is, roughly, the base-2 logarithm of the number of
+possible values that can be produced by the mutator. Note that this is distinct
+from the complexity of a value. If we have a mutator for `usize` that can only
+produce the values `89` and `65`, then the search space complexity of the
+mutator is `1.0` but the complexity of the produced values could be `64.0`. If a
+mutator has a search space complexity of `0.0`, then it is only able to
+produce a single value.
 
- In order to mutate values efficiently, the mutator is able to make use of a
- per-value *cache*. The [`Cache`](Mutator::Cache) contains information associated
- with the value that will make it faster to compute its complexity or apply a
- mutation to it. For a vector, its cache is its total complexity, along with a
- vector of the caches of each of its element.
+## [`Cache`](Mutator::Cache)
 
- ## [`MutationStep`](Mutator::MutationStep)
+In order to mutate values efficiently, the mutator is able to make use of a
+per-value *cache*. The [`Cache`](Mutator::Cache) contains information associated
+with the value that will make it faster to compute its complexity or apply a
+mutation to it. For a vector, its cache is its total complexity, along with a
+vector of the caches of each of its element.
 
- The same values will be passed to the mutator many times, so that it is
- mutated in many different ways. There are different strategies to choose
- what mutation to apply to a value. The first one is to create a list of
- mutation operations, and choose one to apply randomly from this list.
+## [`MutationStep`](Mutator::MutationStep)
 
- However, one may want to have better control over which mutation operation
- is used. For example, if the value to be mutated is of type `Option<T>`,
- then you may want to first mutate it to `None`, and then always mutate it
- to another `Some(t)`. This is where [`MutationStep`](Mutator::MutationStep)
- comes in. The mutationstep is a type you define to allow you to keep track
- of which mutation operation has already been tried. This allows you to
- deterministically apply mutations to a value such that better mutations are
- tried first, and duplicate mutations are avoided.
+The same values will be passed to the mutator many times, so that it is
+mutated in many different ways. There are different strategies to choose
+what mutation to apply to a value. The first one is to create a list of
+mutation operations, and choose one to apply randomly from this list.
 
- It is not always possible to schedule mutations in order. For that reason,
- we have two methods: [`random_mutate`](crate::Mutator::random_mutate) executes
- a random mutation, and [`ordered_mutate`](crate::Mutator::ordered_mutate) uses
- the [`MutationStep`](Mutator::MutationStep) to schedule mutations in order.
- The fuzzing engine only ever uses [`ordered_mutate`](crate::Mutator::ordered_mutate)
- directly, but the former is sometimes necessary to compose mutators together.
+However, one may want to have better control over which mutation operation
+is used. For example, if the value to be mutated is of type `Option<T>`,
+then you may want to first mutate it to `None`, and then always mutate it
+to another `Some(t)`. This is where [`MutationStep`](Mutator::MutationStep)
+comes in. The mutation step is a type you define to allow you to keep track
+of which mutation operation has already been tried. This allows you to
+deterministically apply mutations to a value such that better mutations are
+tried first, and duplicate mutations are avoided.
 
- If you don't want to bother with ordered mutations, that is fine. In that
- case, only implement [`random_mutate`](crate::Mutator::random_mutate) and call it from
- the [`ordered_mutate`](crate::Mutator::ordered_mutate) method.
- ```ignore
- fn random_mutate(&self, value: &mut Value, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
-      // ...
- }
- fn ordered_mutate(&self, value: &mut Value, cache: &mut Self::Cache, step: &mut Self::MutationStep, max_cplx: f64) -> Option<(Self::UnmutateToken, f64)> {
-     Some(self.random_mutate(value, cache, max_cplx))
- }
- ```
+It is not always possible to schedule mutations in order. For that reason,
+we have two methods: [`random_mutate`](crate::Mutator::random_mutate) executes
+a random mutation, and [`ordered_mutate`](crate::Mutator::ordered_mutate) uses
+the [`MutationStep`](Mutator::MutationStep) to schedule mutations in order.
+The fuzzing engine only ever uses [`ordered_mutate`](crate::Mutator::ordered_mutate)
+directly, but the former is sometimes necessary to compose mutators together.
 
- ## Arbitrary
+If you don't want to bother with ordered mutations, that is fine. In that
+case, only implement [`random_mutate`](crate::Mutator::random_mutate) and call it from
+the [`ordered_mutate`](crate::Mutator::ordered_mutate) method.
+```ignore
+fn random_mutate(&self, value: &mut Value, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
+     // ...
+}
+fn ordered_mutate(&self, value: &mut Value, cache: &mut Self::Cache, step: &mut Self::MutationStep, _subvalue_provider: &dyn SubValueProvider, max_cplx: f64) -> Option<(Self::UnmutateToken, f64)> {
+    Some(self.random_mutate(value, cache, max_cplx))
+}
+```
 
- A mutator must also be able to generate new values from nothing. This is what
- the [`random_arbitrary`](crate::Mutator::random_arbitrary) and
- [`ordered_arbitrary`](crate::Mutator::ordered_arbitrary) methods are for. The
- latter one is called by the fuzzer directly and uses an
- [`ArbitraryStep`](Mutator::ArbitraryStep) that can be used to smartly generate
- more interesting values first and avoid duplicates.
+## Arbitrary
 
- ## Unmutate
+A mutator must also be able to generate new values from nothing. This is what
+the [`random_arbitrary`](crate::Mutator::random_arbitrary) and
+[`ordered_arbitrary`](crate::Mutator::ordered_arbitrary) methods are for. The
+latter one is called by the fuzzer directly and uses an
+[`ArbitraryStep`](Mutator::ArbitraryStep) that can be used to smartly generate
+more interesting values first and avoid duplicates.
 
- Finally, it is important to note that values and caches are mutated
- *in-place*. The fuzzer does not clone them before handing them to the
- mutator. Therefore, the mutator also needs to know how to reverse each
- mutation it performed. To do so, each mutation needs to return a token
- describing how to reverse it. The [unmutate](crate::Mutator::unmutate)
- method will later be called with that token to get the original value
- and cache back.
+## Unmutate
 
- For example, if the value is `[[1, 3], [5], [9, 8]]`, the mutator may
- mutate it to `[[1, 3], [5], [9, 1, 8]]` and return the token:
- `Element(2, Remove(1))`, which means that in order to reverse the
- mutation, the element at index 2 has to be unmutated by removing
- its element at index 1. In pseudocode:
+It is important to note that values and caches are mutated
+*in-place*. The fuzzer does not clone them before handing them to the
+mutator. Therefore, the mutator also needs to know how to reverse each
+mutation it performed. To do so, each mutation needs to return a token
+describing how to reverse it. The [unmutate](crate::Mutator::unmutate)
+method will later be called with that token to get the original value
+and cache back.
 
- ```
- use fuzzcheck::Mutator;
+For example, if the value is `[[1, 3], [5], [9, 8]]`, the mutator may
+mutate it to `[[1, 3], [5], [9, 1, 8]]` and return the token:
+`Element(2, Remove(1))`, which means that in order to reverse the
+mutation, the element at index 2 has to be unmutated by removing
+its element at index 1. In pseudocode:
 
+```
+use fuzzcheck::Mutator;
+# use fuzzcheck::subvalue_provider::EmptySubValueProvider;
 # use fuzzcheck::DefaultMutator;
 # let m = bool::default_mutator();
 # let mut value = false;
@@ -117,21 +134,21 @@ use std::path::PathBuf;
 //  cache: c1 (ommitted from example)
 //  step: s1 (ommitted from example)
 
- let (unmutate_token, _cplx) = m.ordered_mutate(&mut value, &mut cache, &mut step, max_cplx).unwrap();
+let (unmutate_token, _cplx) = m.ordered_mutate(&mut value, &mut cache, &mut step, &EmptySubValueProvider, max_cplx).unwrap();
 
- // value = [[1, 3], [5], [9, 1, 8]]
- // token = Element(2, Remove(1))
- // cache = c2
- // step = s2
+// value = [[1, 3], [5], [9, 1, 8]]
+// token = Element(2, Remove(1))
+// cache = c2
+// step = s2
 
- test(&value);
+test(&value);
 
- m.unmutate(&mut value, &mut cache, unmutate_token);
+m.unmutate(&mut value, &mut cache, unmutate_token);
 
- // value = [[1, 3], [5], [9, 8]]
- // cache = c1 (back to original cache)
- // step = s2 (step has not been reversed)
- ```
+// value = [[1, 3], [5], [9, 8]]
+// cache = c1 (back to original cache)
+// step = s2 (step has not been reversed)
+```
 
 When a mutated value is deemed interesting by the fuzzing engine, the method
 [`validate_value`](crate::Mutator::validate_value) is called on it in order to
@@ -143,8 +160,58 @@ will check whether the character is within a certain range.
 Note that in most cases, it is completely fine to never mutate a value’s cache,
 since it is recomputed by [`validate_value`](crate::Mutator::validate_value) when
 needed.
+
+## SubValueProvider
+
+The method `ordered_mutate` takes a [`&dyn SubValueProvider`](crate::SubValueProvider)
+as argument. The purpose of a sub-value provider is to provide the mutator with
+subvalues taken from the fuzzing corpus. If you are familiar with fuzzing
+terminology, then think of the sub-value provider as the structure-aware replacement
+for the “crossover” mutation and the dictionary. Here is how it works:
+
+For each value in the fuzzing corpus, the mutator iterates over each subpart of the
+value by calling [`self.visit_subvalues(value, cache, visit_closure)`](Mutator::visit_subvalues).
+For example, for the value
+```
+struct S {
+    a: usize,
+    b: Option<bool>,
+    c: (Option<bool>, usize)
+}
+let x = S {
+    a: 887236,
+    b: None,
+    c: (Some(true), 10372)
+};
+```
+the `visit_subvalues` method will call the `visit` closure with each subvalue
+and its complexity. For the value `x` above, it will be called with the
+following arguments:
+```ignore
+(&x.a           , 64.0) // 887236
+(&x.b           , 1.0)  // None
+(&x.c           , 66.0) // (Some(true), 10372)
+(&x.c.0         , 2.0)  // Some(true)
+(&x.c.1         , 64.0) // 10372
+(&x.c.0.unwrap(), 1.0)  // true
+```
+
+The fuzzer builds a data structure keeping track of these subvalues and pass it
+to the mutator as a `&dyn SubValueProvider`. The mutator could then use it as
+follows:
+```ignore
+fn ordered_mutate(&self, value: &mut S, cache: &mut Self::Cache, step: &mut Self::Step, subvalue_provider: &dyn SubValueProvider, max_cplx: f64) -> Option<(Self::UnmutateToken, f64)>
+{
+    // let's say we want to replace the value x.c.1 with something taken from the subvalue provider
+    if let Some((new_xc1, new_xc1_cplx)) = subvalue_provider.get_subvalue(TypeId::of::<usize>(), &mut idx, max_xc1_cplx) {
+        let new_xc1 = new_xc1.downcast_ref::<usize>().unwrap().clone(); // guaranteed to succeed
+        value.x.c.1 = new_xc1;
+        // etc.
+    }
+}
+```
 **/
-pub trait Mutator<Value: Clone>: 'static {
+pub trait Mutator<Value: Clone + 'static>: 'static {
     /// Accompanies each value to help compute its complexity and mutate it efficiently.
     type Cache: Clone;
     /// Contains information about what mutations have already been tried.
@@ -157,23 +224,29 @@ pub trait Mutator<Value: Clone>: 'static {
     /// The first [`ArbitraryStep`](Mutator::ArbitraryStep) value to be passed to [`ordered_arbitrary`](crate::Mutator::ordered_arbitrary)
     fn default_arbitrary_step(&self) -> Self::ArbitraryStep;
 
+    /// Quickly verifies that the value conforms to the mutator’s expectations
+    fn is_valid(&self, value: &Value) -> bool;
+
     /// Verifies that the value conforms to the mutator’s expectations and, if it does,
-    /// returns the [`Cache`](Mutator::Cache) with that value.
+    /// returns the [`Cache`](Mutator::Cache) associated with that value.
     fn validate_value(&self, value: &Value) -> Option<Self::Cache>;
 
     /// Returns the first [`MutationStep`](Mutator::MutationStep) associated with the value
     /// and cache.
     fn default_mutation_step(&self, value: &Value, cache: &Self::Cache) -> Self::MutationStep;
 
+    /// The log2 of the number of values that can be produced by this mutator,
+    /// or an approximation of this number (e.g. the number of bits that are
+    /// needed to identify each possible value).
+    ///
+    /// If the mutator can only produce one value, then the return value should
+    /// be equal to 0.0
+    fn global_search_space_complexity(&self) -> f64;
+
     /// The maximum complexity that a value can possibly have.
-    ///
-    /// If the maximum complexity is 0, it means that the mutator
-    /// can only produce one value.
     fn max_complexity(&self) -> f64;
+
     /// The minimum complexity that a value can possibly have.
-    ///
-    /// If the minimum complexity is 0, then the maximum complexity must
-    /// also be 0. It means that the mutator can only produce one value.
     fn min_complexity(&self) -> f64;
 
     /// Computes the complexity of the value.
@@ -218,6 +291,7 @@ pub trait Mutator<Value: Clone>: 'static {
         value: &mut Value,
         cache: &mut Self::Cache,
         step: &mut Self::MutationStep,
+        subvalue_provider: &dyn SubValueProvider,
         max_cplx: f64,
     ) -> Option<(Self::UnmutateToken, f64)>;
 
@@ -234,35 +308,21 @@ pub trait Mutator<Value: Clone>: 'static {
     /// the given [`UnmutateToken`](Mutator::UnmutateToken).
     fn unmutate(&self, value: &mut Value, cache: &mut Self::Cache, t: Self::UnmutateToken);
 
-    type RecursingPartIndex: Clone;
-
-    fn default_recursing_part_index(&self, value: &Value, cache: &Self::Cache) -> Self::RecursingPartIndex;
-    fn recursing_part<'a, T, M>(
-        &self,
-        parent: &M,
-        value: &'a Value,
-        index: &mut Self::RecursingPartIndex,
-    ) -> Option<&'a T>
-    where
-        T: Clone + 'static,
-        M: Mutator<T>;
+    /// Call the given closure on all subvalues and their complexities.
+    fn visit_subvalues<'a>(&self, value: &'a Value, cache: &'a Self::Cache, visit: &mut dyn FnMut(&'a dyn Any, f64));
 }
 
-/**
- * A [Serializer] is used to encode and decode values into bytes.
- *
- * One possible implementation would be to use `serde` to implement
- * both required functions. But we also want to be able to fuzz-test
- * types that are not serializable with `serde`, which is why this
- * trait exists.
-*/
+/// A [Serializer] is used to encode and decode test cases into bytes.
+///
+/// It is used to transfer test cases between the corpus on the file system and the fuzzer’s storage.
 pub trait Serializer {
-    /// The type of the valeu to be serialized
+    /// The type of the value to be serialized
     type Value;
 
     /// The extension of the file containing the serialized value
     fn extension(&self) -> &str;
 
+    #[allow(clippy::wrong_self_convention)]
     /// Deserialize the bytes into the value.
     ///
     /// This method can fail by returning `None`
@@ -274,134 +334,9 @@ pub trait Serializer {
     fn to_data(&self, value: &Self::Value) -> Vec<u8>;
 }
 
-/**
- A trait for types that are basic wrappers over a mutator, such as `Box<M>`.
-
- Such wrapper types automatically implement the [`Mutator`](Mutator) trait.
-*/
-pub trait MutatorWrapper {
-    type Wrapped;
-
-    fn wrapped_mutator(&self) -> &Self::Wrapped;
-}
-
-impl<T: Clone, W, M> Mutator<T> for M
-where
-    M: MutatorWrapper<Wrapped = W>,
-    W: Mutator<T>,
-    Self: 'static,
-{
-    #[doc(hidden)]
-    type Cache = W::Cache;
-    #[doc(hidden)]
-    type MutationStep = W::MutationStep;
-    #[doc(hidden)]
-    type ArbitraryStep = W::ArbitraryStep;
-    #[doc(hidden)]
-    type UnmutateToken = W::UnmutateToken;
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
-        self.wrapped_mutator().default_arbitrary_step()
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn validate_value(&self, value: &T) -> Option<Self::Cache> {
-        self.wrapped_mutator().validate_value(value)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn default_mutation_step(&self, value: &T, cache: &Self::Cache) -> Self::MutationStep {
-        self.wrapped_mutator().default_mutation_step(value, cache)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn max_complexity(&self) -> f64 {
-        self.wrapped_mutator().max_complexity()
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn min_complexity(&self) -> f64 {
-        self.wrapped_mutator().min_complexity()
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn complexity(&self, value: &T, cache: &Self::Cache) -> f64 {
-        self.wrapped_mutator().complexity(value, cache)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(T, f64)> {
-        self.wrapped_mutator().ordered_arbitrary(step, max_cplx)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {
-        self.wrapped_mutator().random_arbitrary(max_cplx)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn ordered_mutate(
-        &self,
-        value: &mut T,
-        cache: &mut Self::Cache,
-        step: &mut Self::MutationStep,
-        max_cplx: f64,
-    ) -> Option<(Self::UnmutateToken, f64)> {
-        self.wrapped_mutator().ordered_mutate(value, cache, step, max_cplx)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn random_mutate(&self, value: &mut T, cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
-        self.wrapped_mutator().random_mutate(value, cache, max_cplx)
-    }
-
-    #[doc(hidden)]
-    type RecursingPartIndex = W::RecursingPartIndex;
-    #[doc(hidden)]
-    #[no_coverage]
-    fn unmutate(&self, value: &mut T, cache: &mut Self::Cache, t: Self::UnmutateToken) {
-        self.wrapped_mutator().unmutate(value, cache, t)
-    }
-    #[doc(hidden)]
-    #[no_coverage]
-    fn default_recursing_part_index(&self, value: &T, cache: &Self::Cache) -> Self::RecursingPartIndex {
-        self.wrapped_mutator().default_recursing_part_index(value, cache)
-    }
-
-    #[doc(hidden)]
-    #[no_coverage]
-    fn recursing_part<'a, V, N>(&self, parent: &N, value: &'a T, index: &mut Self::RecursingPartIndex) -> Option<&'a V>
-    where
-        V: Clone + 'static,
-        N: Mutator<V>,
-    {
-        let m = self.wrapped_mutator();
-        m.recursing_part::<V, N>(parent, value, index)
-    }
-}
-
-impl<M> MutatorWrapper for Box<M> {
-    type Wrapped = M;
-    #[no_coverage]
-    fn wrapped_mutator(&self) -> &Self::Wrapped {
-        self.as_ref()
-    }
-}
-
 /// A [CorpusDelta] describes how to reflect a change in the pool’s content to the corpus on the file system.
 ///
-/// It is used as the return type to [`pool.process(..)`](CompatibleWithSensor::process) where a test case along
+/// It is used as the return type to [`pool.process(..)`](CompatibleWithObservations::process) where a test case along
 /// with its associated sensor observations is given to the pool. Thus, it is always implicitly associated with
 /// a specific pool and test case.
 #[derive(Debug)]
@@ -441,41 +376,44 @@ fuzzer can use to determine the importance of a test case.
 For example, the sensor can record the code coverage triggered by the test case,
 store the source location of a panic, measure the number of allocations made, etc.
 The observations made by a sensor are then assessed by a [Pool], which must be
-explicitly [compatible](CompatibleWithSensor) with the sensor.
+explicitly [compatible](CompatibleWithObservations) with the sensor’s observations.
 */
-pub trait Sensor: SaveToStatsFolder {
-    /**
-    A type that is used to retrieve the observations made by the sensor.
-
-    For example, if the sensor stores only one value of type `u8`, the
-    observation handler can be `&'a mut u8`. But if the observations are retrieved one-by-one, the observation handler may be `&'a mut dyn FnMut(T)`.
-    */
-    type ObservationHandler<'a>;
+pub trait Sensor: SaveToStatsFolder + 'static {
+    type Observations;
 
     /// Signal to the sensor that it should prepare to record observations
     fn start_recording(&mut self);
     /// Signal to the sensor that it should stop recording observations
     fn stop_recording(&mut self);
 
-    /// Access the sensor's observations through the handler
-    fn iterate_over_observations(&mut self, handler: Self::ObservationHandler<'_>);
+    /// Access the sensor's observations
+    fn get_observations(&mut self) -> Self::Observations;
 }
 
+/// A trait implemented by the [statistics of a pool](crate::Pool::Stats)
+///
+/// The types implementing `Stats` must be displayable in the terminal and must be
+/// [convertible to CSV fields](crate::ToCSV). However, note that at the moment some pools
+/// choose to produce empty CSV values for their statistics. Consequently, their statistics
+/// will not be available in the `fuzz/stats/<id>/events.csv` file written by fuzzcheck
+/// at the end of a fuzz test.
+///
+/// Some pools may choose not to display their statistics in the terminal.
 pub trait Stats: Display + ToCSV + 'static {}
 
-/// An object safe trait that combines the methods of the [`Sensor`], [`Pool`], and [`CompatibleWithSensor`] traits.
+/// An object safe trait that combines the methods of the [`Sensor`], [`Pool`], and [`CompatibleWithObservations`] traits.
 ///
 /// While it's often useful to work with the [`Sensor`] and [`Pool`] traits separately, the
 /// fuzzer doesn't actually need to know about the sensor and pool individually. By having
 /// this `SensorAndPool` trait, we can give the fuzzer a `Box<dyn SensorAndPool>` and get rid of
-/// two generic type parameters: `S: Sensor` and `P: Pool + CompatibleWithSensor<S>`.
+/// two generic type parameters: `S: Sensor` and `P: Pool + CompatibleWithObservations<S::Observations>`.
 ///
 /// This is better for compile times and simplifies the implementation of the fuzzer. Users of
 /// `fuzzcheck` should feel free to ignore this trait, as it is arguably more an implementation detail
 /// than a fundamental building block of the fuzzer.
 ///
 /// Currently, there are two types implementing `SensorAndPool`:
-/// 1. `(S, P)` where `S: Sensor` and `P: Pool + CompatibleWithSensor<S>`
+/// 1. `(S, P)` where `S: Sensor` and `P: Pool + CompatibleWithObservations<S::Observations>`
 /// 2. [`AndSensorAndPool`](crate::sensors_and_pools::AndSensorAndPool)
 pub trait SensorAndPool: SaveToStatsFolder {
     fn stats(&self) -> Box<dyn Stats>;
@@ -489,6 +427,7 @@ where
     A: SaveToStatsFolder,
     B: SaveToStatsFolder,
 {
+    #[no_coverage]
     fn save_to_stats_folder(&self) -> Vec<(PathBuf, Vec<u8>)> {
         let mut x = self.0.save_to_stats_folder();
         x.extend(self.1.save_to_stats_folder());
@@ -498,25 +437,27 @@ where
 impl<S, P> SensorAndPool for (S, P)
 where
     S: Sensor,
-    P: CompatibleWithSensor<S>,
+    P: CompatibleWithObservations<S::Observations>,
     S: SaveToStatsFolder,
     P: SaveToStatsFolder,
 {
+    #[no_coverage]
     fn stats(&self) -> Box<dyn Stats> {
         Box::new(self.1.stats())
     }
+    #[no_coverage]
     fn start_recording(&mut self) {
         self.0.start_recording();
     }
-
+    #[no_coverage]
     fn stop_recording(&mut self) {
         self.0.stop_recording();
     }
-
+    #[no_coverage]
     fn process(&mut self, input_id: PoolStorageIndex, complexity: f64) -> Vec<CorpusDelta> {
-        self.1.process(input_id, &mut self.0, complexity)
+        self.1.process(input_id, &self.0.get_observations(), complexity)
     }
-
+    #[no_coverage]
     fn get_random_index(&mut self) -> Option<PoolStorageIndex> {
         self.1.get_random_index()
     }
@@ -571,10 +512,12 @@ pub trait ToCSV {
     fn to_csv_record(&self) -> Vec<CSVField>;
 }
 impl ToCSV for Box<dyn Stats> {
+    #[no_coverage]
     fn csv_headers(&self) -> Vec<CSVField> {
         self.as_ref().csv_headers()
     }
 
+    #[no_coverage]
     fn to_csv_record(&self) -> Vec<CSVField> {
         self.as_ref().to_csv_record()
     }
@@ -585,7 +528,7 @@ A [`Pool`] ranks test cases based on observations recorded by a sensor.
 
 The pool trait is divided into two parts:
 1. [`Pool`] contains general methods that are independent of the sensor used
-2. [`CompatibleWithSensor<Sensor>`] is a subtrait of [`Pool`]. It describes how the pool handles
+2. [`CompatibleWithObservations<O>`] is a subtrait of [`Pool`]. It describes how the pool handles
 observations made by the [`Sensor`].
 */
 pub trait Pool: SaveToStatsFolder {
@@ -601,6 +544,16 @@ pub trait Pool: SaveToStatsFolder {
     /// Most [Pool] implementations will want to prioritise certain test cases
     /// over others based on their associated observations.
     fn get_random_index(&mut self) -> Option<PoolStorageIndex>;
+
+    /// Gives the relative importance of the pool. It must be a positive number.
+    ///
+    /// The weight of the pool is not used by the fuzzer directly, but can be used
+    /// by types such as [`AndPool`](crate::sensors_and_pools::AndPool).
+    ///
+    /// The value is 1.0 by default.
+    fn weight(&self) -> f64 {
+        1.0
+    }
 }
 
 /**
@@ -610,11 +563,11 @@ This trait is separate from [Pool] because a single pool type may handle multipl
 
 It is responsible for judging whether the observations are interesting, and then adding the test case to the pool
 if they are. It communicates to the rest of the fuzzer what test cases were added or removed from the pool via the
-[CorpusDelta] type. This ensures that the right message can be printed to the terminal and that the corpus on the
+[`CorpusDelta`] type. This ensures that the right message can be printed to the terminal and that the corpus on the
 file system, which reflects the content of the pool, can be properly updated.
 */
-pub trait CompatibleWithSensor<S: Sensor>: Pool {
-    fn process(&mut self, input_id: PoolStorageIndex, sensor: &mut S, complexity: f64) -> Vec<CorpusDelta>;
+pub trait CompatibleWithObservations<O>: Pool {
+    fn process(&mut self, input_id: PoolStorageIndex, observations: &O, complexity: f64) -> Vec<CorpusDelta>;
 }
 
 /// A trait for types that want to save their content to the `stats` folder which is created after a fuzzing run.
