@@ -1,8 +1,6 @@
-use fuzzcheck::make_mutator;
 use fuzzcheck::mutators::boxed::BoxMutator;
 use fuzzcheck::mutators::recursive::RecurToMutator;
-use fuzzcheck::DefaultMutator;
-use fuzzcheck::Mutator;
+use fuzzcheck::{make_mutator, DefaultMutator, Mutator};
 
 #[test]
 #[no_coverage]
