@@ -1,10 +1,9 @@
-use fuzzcheck::make_mutator;
+#![allow(clippy::type_complexity)]
 use fuzzcheck::mutators::bool::BoolMutator;
 use fuzzcheck::mutators::boxed::BoxMutator;
 use fuzzcheck::mutators::option::OptionMutator;
 use fuzzcheck::mutators::recursive::{RecurToMutator, RecursiveMutator};
-use fuzzcheck::DefaultMutator;
-use fuzzcheck::Mutator;
+use fuzzcheck::{make_mutator, DefaultMutator, Mutator};
 
 #[derive(Clone, Debug)]
 struct S {
@@ -31,7 +30,7 @@ pub struct R<T> {
     z: Vec<T>,
 }
 make_mutator! {
-    name: RMutator
+    name: RMutator,
     recursive: true,
     default: true,
     type: // repeat the declaration of E
